@@ -25,6 +25,7 @@
 #include <QDebug>
 #include <QFile>
 #include <QImage>
+#include <qmath.h>
 #include <QtToString.h>
 #include <QXmlStreamReader>
 #include <QXmlStreamWriter>
@@ -35,24 +36,33 @@
 
 const int FOUR_BYTES = 4;
 
-CoordSystem::CoordSystem (DocumentAxesPointsRequired documentAxesPointsRequired) :
+CoordSystem::CoordSystem () :
   m_curveAxes (new Curve (AXIS_CURVE_NAME,
                           ColorFilterSettings::defaultFilter (),
                           CurveStyle (LineStyle::defaultAxesCurve(),
-                                      PointStyle::defaultAxesCurve ()))),
-  m_documentAxesPointsRequired (documentAxesPointsRequired)
+                                      PointStyle::defaultAxesCurve ())))
 {
   LOG4CPP_INFO_S ((*mainCat)) << "CoordSystem::CoordSystem";
 
   SettingsForGraph settingsForGraph;
-  QString curveName = settingsForGraph.defaultCurveName (1,
-                                                         DEFAULT_GRAPH_CURVE_NAME);
-  m_curvesGraphs.addGraphCurveAtEnd (Curve (curveName,
-                                            ColorFilterSettings::defaultFilter (),
-                                            CurveStyle (LineStyle::defaultGraphCurve (m_curvesGraphs.numCurves ()),
-                                                        PointStyle::defaultGraphCurve (m_curvesGraphs.numCurves ()))));
 
-  resetSelectedCurveNameIfNecessary ();
+  // Create one curve, or as many curve as specified in the configuration file, whichever is greater
+  for (int indexOneBased = 1; indexOneBased <= settingsForGraph.numberOfCurvesForImport (); indexOneBased++) {
+
+    QString curveName = settingsForGraph.defaultCurveName (indexOneBased,
+                                                           DEFAULT_GRAPH_CURVE_NAME);
+    m_curvesGraphs.addGraphCurveAtEnd (Curve (curveName,
+                                              ColorFilterSettings::defaultFilter (),
+                                              CurveStyle (LineStyle::defaultGraphCurve (m_curvesGraphs.numCurves ()),
+                                                          PointStyle::defaultGraphCurve (m_curvesGraphs.numCurves ()))));
+
+    resetSelectedCurveNameIfNecessary ();
+  }
+}
+
+CoordSystem::~CoordSystem()
+{
+  delete m_curveAxes;
 }
 
 void CoordSystem::addGraphCurveAtEnd (const QString &curveName)
@@ -171,7 +181,8 @@ void CoordSystem::checkAddPointAxis (const QPointF &posScreen,
                                      const QPointF &posGraph,
                                      bool &isError,
                                      QString &errorMessage,
-                                     bool isXOnly)
+                                     bool isXOnly,
+                                     DocumentAxesPointsRequired documentAxesPointsRequired)
 {
   LOG4CPP_INFO_S ((*mainCat)) << "CoordSystem::checkAddPointAxis"
                               << " posScreen=" << QPointFToString (posScreen).toLatin1 ().data ()
@@ -180,7 +191,7 @@ void CoordSystem::checkAddPointAxis (const QPointF &posScreen,
   CallbackCheckAddPointAxis ftor (m_modelCoords,
                                   posScreen,
                                   posGraph,
-                                  m_documentAxesPointsRequired,
+                                  documentAxesPointsRequired,
                                   isXOnly);
 
   Functor2wRet<const QString &, const Point &, CallbackSearchReturn> ftorWithCallback = functor_ret (ftor,
@@ -195,7 +206,8 @@ void CoordSystem::checkEditPointAxis (const QString &pointIdentifier,
                                       const QPointF &posScreen,
                                       const QPointF &posGraph,
                                       bool &isError,
-                                      QString &errorMessage)
+                                      QString &errorMessage,
+                                      DocumentAxesPointsRequired documentAxesPointsRequired)
 {
   LOG4CPP_INFO_S ((*mainCat)) << "CoordSystem::checkEditPointAxis"
                               << " posGraph=" << QPointFToString (posGraph).toLatin1 ().data ();
@@ -204,7 +216,7 @@ void CoordSystem::checkEditPointAxis (const QString &pointIdentifier,
                                    pointIdentifier,
                                    posScreen,
                                    posGraph,
-                                   m_documentAxesPointsRequired);
+                                   documentAxesPointsRequired);
 
   Functor2wRet<const QString &, const Point &, CallbackSearchReturn> ftorWithCallback = functor_ret (ftor,
                                                                                                      &CallbackCheckEditPointAxis::callback);
@@ -293,6 +305,34 @@ void CoordSystem::editPointGraph (bool isX,
                                  transformation);
 }
 
+void CoordSystem::initializeUnsetGridRemovalFromGridDisplay (double version)
+{
+  // In issue #273 a broken dig file was encountered with grid removal values that were apparently
+  // corrupted, from version 4.1. This code was inserted to accomodate that file and other files presumably having
+  // the same issue. Newer versions are assumed to be properly initialized, and this code is not applied
+  // so it does not interfere with properly set values
+
+  if (version < 5) {
+
+    // Most reliable indicator of a problem is very unrealistic values for counts
+    if (m_modelGridRemoval.countX () < 2 ||
+        m_modelGridRemoval.countY () < 2 ||
+        m_modelGridRemoval.countX () > 100 ||
+        m_modelGridRemoval.countY () > 100) {
+
+      // Problem found. Prevent issues later by copying values from m_modelGridDisplay
+      m_modelGridRemoval.setStartX (m_modelGridDisplay.startX ());
+      m_modelGridRemoval.setStartY (m_modelGridDisplay.startY ());
+      m_modelGridRemoval.setStepX (m_modelGridDisplay.stepX ());
+      m_modelGridRemoval.setStepY (m_modelGridDisplay.stepY ());
+      m_modelGridRemoval.setStopX (m_modelGridDisplay.stopX ());
+      m_modelGridRemoval.setStopY (m_modelGridDisplay.stopY ());
+      m_modelGridRemoval.setCountX (signed (m_modelGridDisplay.countX ()));
+      m_modelGridRemoval.setCountY (signed (m_modelGridDisplay.countY ()));
+    }
+  }
+}
+
 bool CoordSystem::isXOnly (const QString &pointIdentifier) const
 {
   return m_curveAxes->isXOnly (pointIdentifier);
@@ -345,7 +385,8 @@ bool CoordSystem::loadCurvesFile(const QString & /* curvesFile */)
 }
 
 void CoordSystem::loadPreVersion6 (QDataStream &str,
-                                   double version)
+                                   double version,
+                                   DocumentAxesPointsRequired &documentAxesPointsRequired)
 {
   LOG4CPP_INFO_S ((*mainCat)) << "CoordSystem::loadPreVersion6";
 
@@ -353,33 +394,33 @@ void CoordSystem::loadPreVersion6 (QDataStream &str,
   double dbl, radius = 0.0;
   QString st;
 
-  m_documentAxesPointsRequired = DOCUMENT_AXES_POINTS_REQUIRED_3;
-
   str >> st; // CurveCmbText selection
   str >> st; // MeasureCmbText selection
   str >> int32;
-  m_modelCoords.setCoordsType((CoordsType) int32);
+  m_modelCoords.setCoordsType(static_cast<CoordsType> (int32));
   if (version >= 3) {
-    str >> (double &) radius;
+    str >> static_cast<double &> (radius);
   }
   m_modelCoords.setOriginRadius(radius);
   str >> int32;
   m_modelCoords.setCoordUnitsRadius(COORD_UNITS_NON_POLAR_THETA_NUMBER);
-  m_modelCoords.setCoordUnitsTheta((CoordUnitsPolarTheta) int32);
+  m_modelCoords.setCoordUnitsTheta(static_cast<CoordUnitsPolarTheta> (int32));
   str >> int32;
-  m_modelCoords.setCoordScaleXTheta((CoordScale) int32);
+  m_modelCoords.setCoordScaleXTheta(static_cast<CoordScale> (int32));
   str >> int32;
-  m_modelCoords.setCoordScaleYRadius((CoordScale) int32);
+  m_modelCoords.setCoordScaleYRadius(static_cast<CoordScale> (int32));
 
   str >> int32;
-  m_modelExport.setDelimiter((ExportDelimiter) int32);
+  m_modelExport.setDelimiter(static_cast<ExportDelimiter> (int32));
   str >> int32;
-  m_modelExport.setLayoutFunctions((ExportLayoutFunctions) int32);
+  m_modelExport.setLayoutFunctions(static_cast<ExportLayoutFunctions> (int32));
   str >> int32;
-  m_modelExport.setPointsSelectionFunctions((ExportPointsSelectionFunctions) int32);
-  m_modelExport.setPointsIntervalUnitsRelations((ExportPointsIntervalUnits) int32);
+  m_modelExport.setPointsSelectionFunctions(static_cast<ExportPointsSelectionFunctions> (int32));
+  m_modelExport.setPointsSelectionRelations(EXPORT_POINTS_SELECTION_RELATIONS_RAW); // Best for maps
+  m_modelExport.setPointsIntervalUnitsFunctions(static_cast<ExportPointsIntervalUnits> (int32));
+  m_modelExport.setPointsIntervalUnitsRelations(static_cast<ExportPointsIntervalUnits> (int32));
   str >> int32;
-  m_modelExport.setHeader((ExportHeader) int32);
+  m_modelExport.setHeader(static_cast<ExportHeader> (int32));
   if (version >= 5.1) {
     str >> st; // X label
     if (m_modelCoords.coordsType() == COORDS_TYPE_CARTESIAN) {
@@ -402,9 +443,9 @@ void CoordSystem::loadPreVersion6 (QDataStream &str,
   str >> int32;
   m_modelGridRemoval.setCountY(int32);
   str >> int32;
-  m_modelGridRemoval.setGridCoordDisableX((GridCoordDisable) int32);
+  m_modelGridRemoval.setGridCoordDisableX(static_cast<GridCoordDisable> (int32));
   str >> int32;
-  m_modelGridRemoval.setGridCoordDisableY((GridCoordDisable) int32);
+  m_modelGridRemoval.setGridCoordDisableY(static_cast<GridCoordDisable> (int32));
   str >> dbl;
   m_modelGridRemoval.setStartX(dbl);
   str >> dbl;
@@ -433,13 +474,13 @@ void CoordSystem::loadPreVersion6 (QDataStream &str,
   str >> int32;
   m_modelGridDisplay.setStable(int32);
   str >> int32;
-  m_modelGridDisplay.setCountX(int32);
+  m_modelGridDisplay.setCountX(unsigned (int32));
   str >> int32;
-  m_modelGridDisplay.setCountY(int32);
+  m_modelGridDisplay.setCountY(unsigned (int32));
   str >> int32;
-  m_modelGridDisplay.setDisableX((GridCoordDisable) int32);
+  m_modelGridDisplay.setDisableX(static_cast<GridCoordDisable> (int32));
   str >> int32;
-  m_modelGridDisplay.setDisableY((GridCoordDisable) int32);
+  m_modelGridDisplay.setDisableY(static_cast<GridCoordDisable> (int32));
   str >> dbl;
   m_modelGridDisplay.setStartX (dbl);
   str >> dbl;
@@ -453,27 +494,29 @@ void CoordSystem::loadPreVersion6 (QDataStream &str,
   str >> dbl;
   m_modelGridDisplay.setStopY (dbl);
 
+  initializeUnsetGridRemovalFromGridDisplay (version);
+
   str >> int32;
   m_modelSegments.setMinLength(int32);
   str >> int32;
   m_modelSegments.setPointSeparation(int32);
   str >> int32;
-  m_modelSegments.setLineWidth(int32);
+  m_modelSegments.setLineWidthActive(int32);
   str >> int32;
-  m_modelSegments.setLineColor((ColorPalette) int32);
+  m_modelSegments.setLineColor(static_cast<ColorPalette> (int32));
 
   str >> int32; // Point separation
   str >> int32;
   m_modelPointMatch.setMaxPointSize(int32);
   str >> int32;
-  m_modelPointMatch.setPaletteColorAccepted((ColorPalette) int32);
+  m_modelPointMatch.setPaletteColorAccepted(static_cast<ColorPalette> (int32));
   str >> int32;
-  m_modelPointMatch.setPaletteColorRejected((ColorPalette) int32);
+  m_modelPointMatch.setPaletteColorRejected(static_cast<ColorPalette> (int32));
   if (version < 4) {
     m_modelPointMatch.setPaletteColorCandidate(COLOR_PALETTE_BLUE);
   } else {
     str >> int32;
-    m_modelPointMatch.setPaletteColorCandidate((ColorPalette) int32);
+    m_modelPointMatch.setPaletteColorCandidate(static_cast<ColorPalette> (int32));
   }
 
   str >> int32; // Discretize method
@@ -488,23 +531,39 @@ void CoordSystem::loadPreVersion6 (QDataStream &str,
   str >> int32; // Value threshold low
   str >> int32; // Value threshold high
 
-  m_curveAxes = new Curve (str);
-  Curve curveScale (str); // Scales are dropped on the floor
+  // Old versions have two Curve objects for 3 point axes and 2 point scales. New version picks one Curve
+  Curve *curveAxesIn = new Curve (str);
+  Curve *curveScaleIn = new Curve (str);
+  if (curveScaleIn->numPoints() == 2) {
+    // Nondefault case is map with scale bar
+    documentAxesPointsRequired = DOCUMENT_AXES_POINTS_REQUIRED_2;
+    delete m_curveAxes;
+    m_curveAxes = curveScaleIn;
+    m_curveAxes->setCurveName (AXIS_CURVE_NAME); // Override existing "Scale" name
+    delete curveAxesIn;
+  } else {
+    // Default case is graph with axes
+    documentAxesPointsRequired = DOCUMENT_AXES_POINTS_REQUIRED_3;
+    delete m_curveAxes;    
+    m_curveAxes = curveAxesIn;
+    delete curveScaleIn;
+  }
   m_curvesGraphs.loadPreVersion6 (str);
 
   // Information from curves and points can affect some data structures that were (mostly) set earlier
-  if (m_curveAxes->numPoints () > 2) {
+  if (m_curveAxes->numPoints () >= documentAxesPointsRequired) {
     m_modelGridRemoval.setStable();
   }
 
   resetSelectedCurveNameIfNecessary ();
 }
 
-void CoordSystem::loadVersion6 (QXmlStreamReader &reader)
+void CoordSystem::loadVersion6 (QXmlStreamReader &reader,
+                                DocumentAxesPointsRequired &documentAxesPointsRequired)
 {
   LOG4CPP_INFO_S ((*mainCat)) << "CoordSystem::loadVersion6";
 
-  m_documentAxesPointsRequired = DOCUMENT_AXES_POINTS_REQUIRED_3;
+  documentAxesPointsRequired = DOCUMENT_AXES_POINTS_REQUIRED_3;
 
   // Import from xml. Loop to end of data or error condition occurs, whichever is first
   while (!reader.atEnd() &&
@@ -528,6 +587,7 @@ void CoordSystem::loadVersion6 (QXmlStreamReader &reader)
       } else if (tag == DOCUMENT_SERIALIZE_COORDS) {
         m_modelCoords.loadXml (reader);
       } else if (tag == DOCUMENT_SERIALIZE_CURVE) {
+        delete m_curveAxes;        
         m_curveAxes = new Curve (reader);
       } else if (tag == DOCUMENT_SERIALIZE_CURVES_GRAPHS) {
         m_curvesGraphs.loadXml (reader);
@@ -539,7 +599,10 @@ void CoordSystem::loadVersion6 (QXmlStreamReader &reader)
         m_modelGeneral.loadXml (reader);
       } else if (tag == DOCUMENT_SERIALIZE_GRID_REMOVAL) {
         m_modelGridRemoval.loadXml (reader);
+      } else if (tag == DOCUMENT_SERIALIZE_GUIDELINES) {
+        m_modelGuideline.loadXml (reader);
       } else if (tag == DOCUMENT_SERIALIZE_IMAGE) {
+        LOG4CPP_ERROR_S ((*mainCat)) << "CoordSystem::loadVersion6 encountered image out of order";
         ENGAUGE_ASSERT (false); // The image should have been read before this method was called
       } else if (tag == DOCUMENT_SERIALIZE_POINT_MATCH) {
         m_modelPointMatch.loadXml (reader);
@@ -559,13 +622,10 @@ void CoordSystem::loadVersion6 (QXmlStreamReader &reader)
   resetSelectedCurveNameIfNecessary ();
 }
 
-void CoordSystem::loadVersions7AndUp (QXmlStreamReader &reader,
-                                      DocumentAxesPointsRequired documentAxesPointsRequired)
+void CoordSystem::loadVersions7AndUp (QXmlStreamReader &reader)
 {
   LOG4CPP_INFO_S ((*mainCat)) << "CoordSystem::loadVersions7AndUp";
 
-  m_documentAxesPointsRequired = documentAxesPointsRequired;
-
   // Import from xml. Loop to end of data or error condition occurs, whichever is first
   while (!reader.atEnd() &&
          !reader.hasError()) {
@@ -588,6 +648,7 @@ void CoordSystem::loadVersions7AndUp (QXmlStreamReader &reader,
       } else if (tag == DOCUMENT_SERIALIZE_COORDS) {
         m_modelCoords.loadXml (reader);
       } else if (tag == DOCUMENT_SERIALIZE_CURVE) {
+        delete m_curveAxes;                
         m_curveAxes = new Curve (reader);
       } else if (tag == DOCUMENT_SERIALIZE_CURVES_GRAPHS) {
         m_curvesGraphs.loadXml (reader);
@@ -601,7 +662,10 @@ void CoordSystem::loadVersions7AndUp (QXmlStreamReader &reader,
         m_modelGridDisplay.loadXml (reader);
       } else if (tag == DOCUMENT_SERIALIZE_GRID_REMOVAL) {
         m_modelGridRemoval.loadXml (reader);
+      } else if (tag == DOCUMENT_SERIALIZE_GUIDELINES) {
+        m_modelGuideline.loadXml (reader);
       } else if (tag == DOCUMENT_SERIALIZE_IMAGE) {
+        LOG4CPP_ERROR_S ((*mainCat)) << "CoordSystem::loadVersions7AndUp encountered image out of order";        
         ENGAUGE_ASSERT (false); // The image should have been read before this method was called
       } else if (tag == DOCUMENT_SERIALIZE_POINT_MATCH) {
         m_modelPointMatch.loadXml (reader);
@@ -669,6 +733,11 @@ DocumentModelGridRemoval CoordSystem::modelGridRemoval() const
   return m_modelGridRemoval;
 }
 
+DocumentModelGuideline CoordSystem::modelGuideline() const
+{
+  return m_modelGuideline;
+}
+
 DocumentModelPointMatch CoordSystem::modelPointMatch() const
 {
   return m_modelPointMatch;
@@ -685,7 +754,7 @@ void CoordSystem::movePoint (const QString &pointIdentifier,
   QString curveName = Point::curveNameFromPointIdentifier (pointIdentifier);
 
   Curve *curve = curveForCurveName (curveName);
-  ENGAUGE_ASSERT (curve != 0);
+  ENGAUGE_CHECK_PTR (curve);
   curve->movePoint (pointIdentifier,
                     deltaScreen);
 }
@@ -703,7 +772,7 @@ int CoordSystem::nextOrdinalForCurve (const QString &curveName) const
     m_curvesGraphs.iterateThroughCurvesPoints (ftorWithCallback);
   }
 
-  return ftor.nextOrdinal ();
+  return qFloor (ftor.nextOrdinal ());
 }
 
 QPointF CoordSystem::positionGraph (const QString &pointIdentifier) const
@@ -761,6 +830,8 @@ void CoordSystem::printStream (QString indentation,
                                   str);
   m_modelGridRemoval.printStream (indentation,
                                   str);
+  m_modelGuideline.printStream (indentation,
+                                str);
   m_modelPointMatch.printStream (indentation,
                                  str);
   m_modelSegments.printStream (indentation,
@@ -801,7 +872,7 @@ void CoordSystem::removePointsInCurvesGraphs (CurvesGraphs &curvesGraphs)
 void CoordSystem::resetSelectedCurveNameIfNecessary ()
 {
   if (m_selectedCurveName.isEmpty () ||
-      curveForCurveName (m_selectedCurveName) == 0) {
+      curveForCurveName (m_selectedCurveName) == nullptr) {
 
     // Selected curve name is empty, or the curve has been removed so we pick another. The first is arbitrarily picked
     m_selectedCurveName = m_curvesGraphs.curvesGraphsNames().first();
@@ -821,6 +892,7 @@ void CoordSystem::saveXml (QXmlStreamWriter &writer) const
   m_modelAxesChecker.saveXml (writer);
   m_modelGridDisplay.saveXml (writer);
   m_modelGridRemoval.saveXml (writer);
+  m_modelGuideline.saveXml (writer);
   m_modelPointMatch.saveXml (writer);
   m_modelSegments.saveXml (writer);
   m_curveAxes->saveXml (writer);
@@ -837,10 +909,7 @@ void CoordSystem::setCurveAxes (const Curve &curveAxes)
 {
   LOG4CPP_INFO_S ((*mainCat)) << "CoordSystem::setCurveAxes";
 
-  if (m_curveAxes != 0) {
-    delete m_curveAxes;
-    m_curveAxes = 0;
-  }
+  delete m_curveAxes;
 
   m_curveAxes = new Curve (curveAxes);
 }
@@ -920,6 +989,11 @@ void CoordSystem::setModelGridRemoval(const DocumentModelGridRemoval &modelGridR
   m_modelGridRemoval = modelGridRemoval;
 }
 
+void CoordSystem::setModelGuideline(const DocumentModelGuideline &modelGuideline)
+{
+  m_modelGuideline = modelGuideline;
+}
+
 void CoordSystem::setModelPointMatch(const DocumentModelPointMatch &modelPointMatch)
 {
   m_modelPointMatch = modelPointMatch;
