@@ -4,6 +4,7 @@
  * LICENSE or go to gnu.org/licenses for details. Distribution requires prior written permission.     *
  ******************************************************************************************************/
 
+#include "ButtonWhatsThis.h"
 #include "CmdMediator.h"
 #include "CmdSettingsColorFilter.h"
 #include "ColorFilter.h"
@@ -25,22 +26,24 @@
 #include <QPixmap>
 #include <QRadioButton>
 #include <QRgb>
+#include <QWhatsThis>
 #include "ViewPreview.h"
 #include "ViewProfile.h"
 #include "ViewProfileDivider.h"
 #include "ViewProfileScale.h"
 
 const int MINIMUM_DIALOG_WIDTH_COLOR_FILTER = 640;
+const int MINIMUM_HEIGHT = 500;
 
 DlgSettingsColorFilter::DlgSettingsColorFilter(MainWindow &mainWindow) :
   DlgSettingsAbstractBase (tr ("Color Filter"),
                            "DlgSettingsColorFilter",
                            mainWindow),
-  m_scenePreview (0),
-  m_viewPreview (0),
-  m_filterThread (0),
-  m_modelColorFilterBefore (0),
-  m_modelColorFilterAfter (0)
+  m_scenePreview (nullptr),
+  m_viewPreview (nullptr),
+  m_filterThread (nullptr),
+  m_modelColorFilterBefore (nullptr),
+  m_modelColorFilterAfter (nullptr)
 {
   LOG4CPP_INFO_S ((*mainCat)) << "DlgSettingsColorFilter::DlgSettingsColorFilter";
 
@@ -52,13 +55,15 @@ DlgSettingsColorFilter::DlgSettingsColorFilter(MainWindow &mainWindow) :
 DlgSettingsColorFilter::~DlgSettingsColorFilter()
 {
   LOG4CPP_INFO_S ((*mainCat)) << "DlgSettingsColorFilter::~DlgSettingsColorFilter";
+
+  delete m_filterThread;
 }
 
 void DlgSettingsColorFilter::createControls (QGridLayout *layout, int &row)
 {
   LOG4CPP_INFO_S ((*mainCat)) << "DlgSettingsColorFilter::createControls";
 
-  QLabel *labelCurve = new QLabel (tr ("Curve Name:"));
+  QLabel *labelCurve = new QLabel (QString ("%1:").arg (tr ("Curve Name")));
   layout->addWidget (labelCurve, row++, 1);
 
   m_cmbCurveName = new QComboBox ();
@@ -66,7 +71,7 @@ void DlgSettingsColorFilter::createControls (QGridLayout *layout, int &row)
   connect (m_cmbCurveName, SIGNAL (activated (const QString &)), this, SLOT (slotCurveName (const QString &))); // activated() ignores code changes
   layout->addWidget (m_cmbCurveName, row++, 1);
 
-  QLabel *labelProfile = new QLabel (tr ("Filter mode:"));
+  QLabel *labelProfile = new QLabel (QString ("%1:").arg (tr ("Filter mode")));
   layout->addWidget (labelProfile, row++, 1);
 
   m_btnIntensity = new QRadioButton (colorFilterModeToString (COLOR_FILTER_MODE_INTENSITY));
@@ -179,12 +184,18 @@ QWidget *DlgSettingsColorFilter::createSubPanel ()
   layout->setColumnStretch(3, 1); // Profile
   layout->setColumnMinimumWidth(4, EMPTY_COLUMN_WIDTH); // Empty column
   layout->setColumnStretch(4, 0);
+\
+  int row = 0;
+  createWhatsThis (layout,
+                   m_btnWhatsThis,
+                   row++,
+                   4);
 
-  int rowLeft = 0, rowRight = 0;
+  int rowLeft = row, rowRight = row;
   createControls (layout, rowLeft);
   createProfileAndScale (layout, rowRight);
 
-  int row = qMax (rowLeft, rowRight);
+  row = qMax (rowLeft, rowRight);
   createPreview (layout, row);
 
   return subPanel;
@@ -200,7 +211,7 @@ QRgb DlgSettingsColorFilter::createThread ()
   QRgb rgbBackground = filter.marginColor(&image);
 
   // Only create thread once
-  if (m_filterThread == 0) {
+  if (m_filterThread == nullptr) {
 
     m_filterThread = new DlgFilterThread (cmdMediator().document().pixmap(),
                                           rgbBackground,
@@ -231,12 +242,8 @@ void DlgSettingsColorFilter::load (CmdMediator &cmdMediator)
   setCmdMediator (cmdMediator);
 
   // Flush old data
-  if (m_modelColorFilterBefore != 0) {
-    delete m_modelColorFilterBefore;
-  }
-  if (m_modelColorFilterAfter != 0) {
-    delete m_modelColorFilterAfter;
-  }
+  delete m_modelColorFilterBefore;
+  delete m_modelColorFilterAfter;
 
   // Save new data
   m_modelColorFilterBefore = new DocumentModelColorFilter (cmdMediator.document().modelColorFilter());
@@ -268,7 +275,7 @@ void DlgSettingsColorFilter::loadForCurveName()
   QString curveName = m_cmbCurveName->currentText();
 
   // Skip if everything is not set up yet
-  if (!curveName.isEmpty () && m_modelColorFilterAfter != 0) {
+  if (!curveName.isEmpty () && m_modelColorFilterAfter != nullptr) {
 
     // Populate controls
     ColorFilterMode colorFilterMode = m_modelColorFilterAfter->colorFilterMode(curveName);
@@ -280,7 +287,8 @@ void DlgSettingsColorFilter::loadForCurveName()
 
     m_scenePreview->clear();
     m_imagePreview = cmdMediator().document().pixmap().toImage();
-    m_scenePreview->addPixmap (QPixmap::fromImage (m_imagePreview));
+    addPixmap (*m_scenePreview,
+               QPixmap::fromImage (m_imagePreview));
 
     QRgb rgbBackground = createThread ();
     m_scale->setBackgroundColor (rgbBackground);
@@ -290,6 +298,13 @@ void DlgSettingsColorFilter::loadForCurveName()
   }
 }
 
+void DlgSettingsColorFilter::setSmallDialogs(bool smallDialogs)
+{
+  if (!smallDialogs) {
+    setMinimumHeight (MINIMUM_HEIGHT);
+  }
+}
+
 void DlgSettingsColorFilter::slotCurveName(const QString & /* curveName */)
 {
   LOG4CPP_INFO_S ((*mainCat)) << "DlgSettingsColorFilter::slotCurveName";
@@ -300,14 +315,14 @@ void DlgSettingsColorFilter::slotCurveName(const QString & /* curveName */)
 void DlgSettingsColorFilter::slotDividerHigh (double xCenter)
 {
   m_modelColorFilterAfter->setHigh (m_cmbCurveName->currentText(),
-                                    xCenter / (double) PROFILE_SCENE_WIDTH ());
+                                    xCenter / double (PROFILE_SCENE_WIDTH ()));
   updatePreview();
 }
 
 void DlgSettingsColorFilter::slotDividerLow (double xCenter)
 {
   m_modelColorFilterAfter->setLow (m_cmbCurveName->currentText(),
-                                   xCenter / (double) PROFILE_SCENE_WIDTH ());
+                                   xCenter / double (PROFILE_SCENE_WIDTH ()));
   updatePreview();
 }
 
@@ -373,7 +388,8 @@ void DlgSettingsColorFilter::slotTransferPiece (int xLeft,
   delete itemPixmap;
 
   // Save new pixmap. Only visible change should be the area covered by the pixels in image
-  m_scenePreview->addPixmap (QPixmap::fromImage (m_imagePreview));
+  addPixmap (*m_scenePreview,
+             QPixmap::fromImage (m_imagePreview));
 }
 
 void DlgSettingsColorFilter::slotValue ()
@@ -386,6 +402,11 @@ void DlgSettingsColorFilter::slotValue ()
   updatePreview();
 }
 
+void DlgSettingsColorFilter::slotWhatsThis ()
+{
+  QWhatsThis::enterWhatsThisMode();
+}
+
 void DlgSettingsColorFilter::updateHistogram()
 {
   LOG4CPP_INFO_S ((*mainCat)) << "DlgSettingsColorFilter::updateHistogram";
@@ -403,7 +424,7 @@ void DlgSettingsColorFilter::updateHistogram()
   // Start with original image
   QImage image = cmdMediator().document().pixmap().toImage();
 
-  double *histogramBins = new double [ColorFilterHistogram::HISTOGRAM_BINS ()];
+  double *histogramBins = new double [unsigned (ColorFilterHistogram::HISTOGRAM_BINS ())];
 
   ColorFilter filter;
   ColorFilterHistogram filterHistogram;
@@ -417,23 +438,25 @@ void DlgSettingsColorFilter::updateHistogram()
   // Draw histogram, normalizing so highest peak exactly fills the vertical range. Log scale is used
   // so smaller peaks do not disappear
   double logMaxBinCount = qLn (maxBinCount);
-  for (int bin = 1; bin < ColorFilterHistogram::HISTOGRAM_BINS (); bin++) {
+  if (qAbs (logMaxBinCount) > 0) { // Will not have divide by zero from logMaxBinCount below
+    for (int bin = 1; bin < ColorFilterHistogram::HISTOGRAM_BINS (); bin++) {
 
-    double x0 = PROFILE_SCENE_WIDTH () * (bin - 1.0) / (ColorFilterHistogram::HISTOGRAM_BINS () - 1.0);
+      double x0 = PROFILE_SCENE_WIDTH () * (bin - 1.0) / (ColorFilterHistogram::HISTOGRAM_BINS () - 1.0);
 
-    // Map logPixelCount through 0 to 0 through PROFILE_SCENE_HEIGHT-1, using log scale
-    double count0 = 1.0 + histogramBins [bin - 1];
-    double y0 = (PROFILE_SCENE_HEIGHT () - 1.0) * (1.0 - qLn (count0) / logMaxBinCount);
+      // Map logPixelCount through 0 to 0 through PROFILE_SCENE_HEIGHT-1, using log scale
+      double count0 = 1.0 + histogramBins [bin - 1];
+      double y0 = (PROFILE_SCENE_HEIGHT () - 1.0) * (1.0 - qLn (count0) / logMaxBinCount);
 
-    double x1 = PROFILE_SCENE_WIDTH () * (bin - 0.0) / (ColorFilterHistogram::HISTOGRAM_BINS () - 1.0);
+      double x1 = PROFILE_SCENE_WIDTH () * (bin - 0.0) / (ColorFilterHistogram::HISTOGRAM_BINS () - 1.0);
 
-    // Map logPixelCount through 0 to 0 through PROFILE_SCENE_HEIGHT-1, using log scale
-    double count1 = 1.0 + histogramBins [bin];
-    double y1 = (PROFILE_SCENE_HEIGHT () - 1.0) * (1.0 - qLn (count1) / logMaxBinCount);
+      // Map logPixelCount through 0 to 0 through PROFILE_SCENE_HEIGHT-1, using log scale
+      double count1 = 1.0 + histogramBins [bin];
+      double y1 = (PROFILE_SCENE_HEIGHT () - 1.0) * (1.0 - qLn (count1) / logMaxBinCount);
 
-    QGraphicsLineItem *line = new QGraphicsLineItem (x0, y0, x1, y1);
-    line->setPen (QPen (QBrush (Qt::black), PEN_WIDTH));
-    m_sceneProfile->addItem (line);
+      QGraphicsLineItem *line = new QGraphicsLineItem (x0, y0, x1, y1);
+      line->setPen (QPen (QBrush (Qt::black), PEN_WIDTH));
+      m_sceneProfile->addItem (line);
+    }
   }
 
   // Create low and high dividers
@@ -441,13 +464,13 @@ void DlgSettingsColorFilter::updateHistogram()
                                         *m_viewProfile,
                                         PROFILE_SCENE_WIDTH (),
                                         PROFILE_SCENE_HEIGHT (),
-                                        PROFILE_SCENE_HEIGHT () * 2.0 / 3.0,
+                                        qFloor (PROFILE_SCENE_HEIGHT () * 2.0 / 3.0),
                                         true);
   m_dividerHigh = new ViewProfileDivider(*m_sceneProfile,
                                          *m_viewProfile,
                                          PROFILE_SCENE_HEIGHT (),
                                          PROFILE_SCENE_WIDTH (),
-                                         PROFILE_SCENE_HEIGHT () / 3.0,
+                                         qFloor (PROFILE_SCENE_HEIGHT () / 3.0),
                                          false);
 
   // Connect the dividers to each other since the shaded areas depend on both divides when low divider is
@@ -491,11 +514,12 @@ void DlgSettingsColorFilter::updateHistogram()
 
   } else {
 
+    LOG4CPP_ERROR_S ((*mainCat)) << "DlgSettingsColorFilter::updateHistogram unknown button pressed";
     ENGAUGE_ASSERT (false);
 
   }
 
-  free (histogramBins);
+  delete[] histogramBins;
 }
 
 void DlgSettingsColorFilter::updatePreview ()
