@@ -4,6 +4,7 @@
  * LICENSE or go to gnu.org/licenses for details. Distribution requires prior written permission.     *
  ******************************************************************************************************/
 
+#include "ButtonWhatsThis.h"
 #include "CallbackBoundingRects.h"
 #include "CmdMediator.h"
 #include "CmdSettingsCoords.h"
@@ -26,10 +27,12 @@
 #include <QGraphicsScene>
 #include <QLabel>
 #include <QLineEdit>
+#include <qmath.h>
 #include <QPalette>
 #include <QRadioButton>
 #include <QStackedWidget>
 #include <QVBoxLayout>
+#include <QWhatsThis>
 #include "Transformation.h"
 #include "ViewPreview.h"
 
@@ -61,28 +64,26 @@ const double YCENTER = (CARTESIAN_COORD_MIN + CARTESIAN_COORD_MAX) / 2.0;
 const double LINE_WIDTH_THIN = 0.0;
 const double LINE_WIDTH_THICK = 2.0;
 
-const double PI = 3.1415926535;
-const double DEG_2_RAD = PI / 180.0;
-
 const int FONT_SIZE = 6;
 
 const double POWER_FOR_LOG = 10.0; // Need a larger power (certainly more than e) to make log gradient obvious
 
 const int MINIMUM_DIALOG_WIDTH_COORDS = 800;
+const int MINIMUM_HEIGHT = 640;
 
 DlgSettingsCoords::DlgSettingsCoords(MainWindow &mainWindow) :
   DlgSettingsAbstractBase (tr ("Coordinates"),
                            "DlgSettingsCoords",
                            mainWindow),
-  m_btnCartesian (0),
-  m_btnPolar (0),
-  m_validatorOriginRadius (0),
-  m_cmbDate (0),
-  m_cmbTime (0),
-  m_scenePreview (0),
-  m_viewPreview (0),
-  m_modelCoordsBefore (0),
-  m_modelCoordsAfter (0)
+  m_btnCartesian (nullptr),
+  m_btnPolar (nullptr),
+  m_validatorOriginRadius (nullptr),
+  m_cmbDate (nullptr),
+  m_cmbTime (nullptr),
+  m_scenePreview (nullptr),
+  m_viewPreview (nullptr),
+  m_modelCoordsBefore (nullptr),
+  m_modelCoordsAfter (nullptr)
 {
   LOG4CPP_INFO_S ((*mainCat)) << "DlgSettingsCoords::DlgSettingsCoords";
 
@@ -102,7 +103,7 @@ void DlgSettingsCoords::annotateAngles (const QFont &defaultFont) {
   for (int direction = 0; direction < 4; direction++) {
 
     QString angle;
-    CoordUnitsPolarTheta thetaUnits = (CoordUnitsPolarTheta) m_cmbXThetaUnits->currentData().toInt();
+    CoordUnitsPolarTheta thetaUnits = static_cast<CoordUnitsPolarTheta> (m_cmbXThetaUnits->currentData().toInt());
 
     switch (thetaUnits) {
       case COORD_UNITS_POLAR_THETA_DEGREES:
@@ -184,10 +185,13 @@ void DlgSettingsCoords::annotateRadiusAtOrigin(const QFont &defaultFont) {
                       YCENTER);
 }
 
-QRectF DlgSettingsCoords::boundingRectGraph (CmdMediator &cmdMediator,
-                                             bool &isEmpty) const
+void DlgSettingsCoords::boundingRectGraph (CmdMediator &cmdMediator,
+                                           bool &isEmpty,
+                                           QPointF &boundingRectGraphMin,
+                                           QPointF &boundingRectGraphMax) const
 {
-  CallbackBoundingRects ftor (mainWindow().transformation());
+  CallbackBoundingRects ftor (cmdMediator.document().documentAxesPointsRequired(),
+                              mainWindow().transformation());
 
   Functor2wRet<const QString &, const Point&, CallbackSearchReturn> ftorWithCallback = functor_ret (ftor,
                                                                                                     &CallbackBoundingRects::callback);
@@ -202,7 +206,8 @@ QRectF DlgSettingsCoords::boundingRectGraph (CmdMediator &cmdMediator,
     cmdMediator.iterateThroughCurvesPointsGraphs (ftorWithCallback);
   }
 
-  return ftor.boundingRectGraph(isEmpty);
+  boundingRectGraphMin = ftor.boundingRectGraphMin (isEmpty);
+  boundingRectGraphMax = ftor.boundingRectGraphMax (isEmpty);
 }
 
 void DlgSettingsCoords::createDateTime (QGridLayout *layout,
@@ -210,7 +215,7 @@ void DlgSettingsCoords::createDateTime (QGridLayout *layout,
 {
   LOG4CPP_INFO_S ((*mainCat)) << "DlgSettingsCoords::createDateTime";
 
-  QLabel *label = new QLabel(tr ("Date/Time:"));
+  QLabel *label = new QLabel(QString ("%1:").arg (tr ("Date/Time")));
   layout->addWidget (label, row, 1);
 
   QWidget *widgetCombos = new QWidget;
@@ -240,7 +245,12 @@ void DlgSettingsCoords::createGroupCoordsType (QGridLayout *layout,
   LOG4CPP_INFO_S ((*mainCat)) << "DlgSettingsCoords::createGroupCoordsType";
 
   m_boxCoordsType = new QGroupBox(tr ("Coordinates Types"));
-  layout->addWidget (m_boxCoordsType, row++, 1, 1, 2);
+  layout->addWidget (m_boxCoordsType, row, 1, 1, 2);
+
+  createWhatsThis (layout,
+                   m_btnWhatsThis,
+                   row++,
+                   3);
 
   QVBoxLayout *layoutGroup = new QVBoxLayout (m_boxCoordsType);
 
@@ -272,7 +282,7 @@ void DlgSettingsCoords::createGroupXTheta (QGridLayout *layout,
   m_boxXTheta->setLayout (layoutXTheta);
   int rowGroup = 0;
 
-  QLabel *labelScale = new QLabel (tr ("Scale:"));
+  QLabel *labelScale = new QLabel (QString ("%1:").arg (tr ("Scale")));
   layoutXTheta->addWidget (labelScale, rowGroup++, COLUMN_0);
 
   m_xThetaLinear = new QRadioButton (tr ("Linear"), m_boxXTheta);
@@ -287,7 +297,7 @@ void DlgSettingsCoords::createGroupXTheta (QGridLayout *layout,
   connect (m_xThetaLog, SIGNAL (released ()), this, SLOT (slotXThetaLog()));
   layoutXTheta->addWidget (m_xThetaLog, rowGroup++, COLUMN_0);
 
-  QLabel *labelThetaUnits = new QLabel(tr ("Units:"));
+  QLabel *labelThetaUnits = new QLabel(QString ("%1:").arg (tr ("Units")));
   layoutXTheta->addWidget (labelThetaUnits, rowGroup++, COLUMN_0);
 
   m_cmbXThetaUnits = new QComboBox;
@@ -307,7 +317,7 @@ void DlgSettingsCoords::createGroupYRadius (QGridLayout *layout,
   m_boxYRadius->setLayout (layoutYRadius);
   int rowGroup = 0;
 
-  QLabel *labelScale = new QLabel (tr ("Scale:"));
+  QLabel *labelScale = new QLabel (QString ("%1:").arg (tr ("Scale")));
   layoutYRadius->addWidget (labelScale, rowGroup++, COLUMN_0);
 
   m_yRadiusLinear = new QRadioButton (tr ("Linear"), m_boxYRadius);
@@ -315,7 +325,7 @@ void DlgSettingsCoords::createGroupYRadius (QGridLayout *layout,
   connect (m_yRadiusLinear, SIGNAL(released()), this, SLOT (slotYRadiusLinear()));
   layoutYRadius->addWidget (m_yRadiusLinear, rowGroup, COLUMN_0);
 
-  QLabel *labelOriginRadius = new QLabel(tr ("Origin radius value:"));
+  QLabel *labelOriginRadius = new QLabel(QString ("%1:").arg (tr ("Origin radius value")));
   layoutYRadius->addWidget (labelOriginRadius, rowGroup++, COLUMN_1);
 
   m_yRadiusLog = new QRadioButton (tr ("Log"), m_boxYRadius);
@@ -332,7 +342,7 @@ void DlgSettingsCoords::createGroupYRadius (QGridLayout *layout,
   connect (m_editOriginRadius, SIGNAL (textChanged (const QString &)), this, SLOT (slotPolarOriginRadius(const QString &)));
   layoutYRadius->addWidget (m_editOriginRadius, rowGroup++, COLUMN_1);
 
-  QLabel *labelUnits = new QLabel(tr ("Units:"));
+  QLabel *labelUnits = new QLabel(QString ("%1:").arg (tr ("Units")));
   layoutYRadius->addWidget (labelUnits, rowGroup++, COLUMN_0);
   
   m_cmbYRadiusUnits = new QComboBox;
@@ -369,6 +379,7 @@ QWidget *DlgSettingsCoords::createSubPanel ()
   LOG4CPP_INFO_S ((*mainCat)) << "DlgSettingsCoords::createSubPanel";
 
   QWidget *subPanel = new QWidget ();
+
   QGridLayout *layout = new QGridLayout (subPanel);
   subPanel->setLayout (layout);
 
@@ -518,8 +529,8 @@ void DlgSettingsCoords::drawPolarTheta ()
   bool isAxis = true;
   for (int step = 0; step < NUM_COORD_STEPS; step++) {
     double theta = POLAR_THETA_MIN + step * POLAR_THETA_STEP;
-    double x = POLAR_RADIUS * cos (theta * DEG_2_RAD);
-    double y = POLAR_RADIUS * sin (theta * DEG_2_RAD);
+    double x = POLAR_RADIUS * cos (qDegreesToRadians (theta));
+    double y = POLAR_RADIUS * sin (qDegreesToRadians (theta));
     QGraphicsLineItem *line = m_scenePreview->addLine (XCENTER, YCENTER, XCENTER + x, YCENTER + y);
     bool isHighlighted = (step % STEPS_PER_CYCLE == 0);
     line->setPen(QPen (QBrush (isHighlighted ? Qt::gray : Qt::lightGray),
@@ -555,22 +566,21 @@ void DlgSettingsCoords::load (CmdMediator &cmdMediator)
 
   // Remove if coordinates are log so later constraints can be applied
   bool isEmpty;
-  QRectF rectGraph = boundingRectGraph (cmdMediator,
-                                        isEmpty);
-  bool xThetaGoesNegative = !isEmpty && (rectGraph.x() <= 0);
-  bool yRGoesNegative = !isEmpty && (rectGraph.y() <= 0);
+  QPointF boundingRectGraphMin, boundingRectGraphMax;
+  boundingRectGraph (cmdMediator,
+                     isEmpty,
+                     boundingRectGraphMin,
+                     boundingRectGraphMax);
+  bool xThetaGoesNegative = !isEmpty && (boundingRectGraphMin.x() <= 0);
+  bool yRGoesNegative = !isEmpty && (boundingRectGraphMin.y() <= 0);
   m_xThetaLinear->setEnabled (!xThetaGoesNegative);
   m_xThetaLog->setEnabled (!xThetaGoesNegative);
   m_yRadiusLinear->setEnabled (!yRGoesNegative);
   m_yRadiusLog->setEnabled (!yRGoesNegative);
 
   // Flush old data
-  if (m_modelCoordsBefore != 0) {
-    delete m_modelCoordsBefore;
-  }
-  if (m_modelCoordsAfter != 0) {
-    delete m_modelCoordsAfter;
-  }
+  delete m_modelCoordsBefore;
+  delete m_modelCoordsAfter;
 
   // Save new data
   m_modelCoordsBefore = new DocumentModelCoords (cmdMediator.document().modelCoords());
@@ -665,7 +675,8 @@ void DlgSettingsCoords::loadComboBoxUnitsNonPolar (QComboBox &cmb,
   ENGAUGE_ASSERT (cmb.count() == NUM_COORD_UNITS_NON_POLAR_THETA);
 
   cmb.setWhatsThis (QString (tr ("Numbers have the simplest and most general format.\n\n"
-                                 "Date and time values have date and/or time components.\n\n"
+                                 "Date and time values have date and/or time components, with dates between 1901 and 2105 on "
+                                 "64 bit operating systems.\n\n"
                                  "Degrees Minutes Seconds (DDD MM SS.S) format uses two integer number for degrees and minutes, and a real number for "
                                  "seconds. There are 60 seconds per minute. During input, spaces must be inserted between the three numbers.")));
 
@@ -712,10 +723,10 @@ void DlgSettingsCoords::loadComboBoxUnitsPolar (QComboBox &cmb,
 
 void DlgSettingsCoords::resetSceneRectangle ()
 {
-  QRect rect (CARTESIAN_COORD_MIN - CARTESIAN_COORD_STEP / 2.0,
-              CARTESIAN_COORD_MIN - CARTESIAN_COORD_STEP / 2.0,
-              CARTESIAN_COORD_MAX - CARTESIAN_COORD_MIN + CARTESIAN_COORD_STEP,
-              CARTESIAN_COORD_MAX - CARTESIAN_COORD_MIN + CARTESIAN_COORD_STEP);
+  QRect rect (qFloor (CARTESIAN_COORD_MIN - CARTESIAN_COORD_STEP / 2.0),
+              qFloor (CARTESIAN_COORD_MIN - CARTESIAN_COORD_STEP / 2.0),
+              qFloor (CARTESIAN_COORD_MAX - CARTESIAN_COORD_MIN + CARTESIAN_COORD_STEP),
+              qFloor (CARTESIAN_COORD_MAX - CARTESIAN_COORD_MIN + CARTESIAN_COORD_STEP));
 
   QGraphicsRectItem *itemPerimeter = new QGraphicsRectItem(rect);
   itemPerimeter->setVisible(false);
@@ -723,6 +734,13 @@ void DlgSettingsCoords::resetSceneRectangle ()
   m_viewPreview->centerOn (QPointF (0.0, 0.0));
 }
 
+void DlgSettingsCoords::setSmallDialogs(bool smallDialogs)
+{
+  if (!smallDialogs) {
+    setMinimumHeight (MINIMUM_HEIGHT);
+  }
+}
+
 void DlgSettingsCoords::slotCartesianPolar (bool)
 {
   LOG4CPP_INFO_S ((*mainCat)) << "DlgSettingsCoords::slotCartesian";
@@ -741,7 +759,7 @@ void DlgSettingsCoords::slotDate(const QString &)
 {
   LOG4CPP_INFO_S ((*mainCat)) << "DlgSettingsCoords::slotDate";
 
-  CoordUnitsDate coordUnits = (CoordUnitsDate) m_cmbDate->currentData ().toInt();
+  CoordUnitsDate coordUnits = static_cast<CoordUnitsDate> (m_cmbDate->currentData ().toInt());
   m_modelCoordsAfter->setCoordUnitsDate(coordUnits);
   updateControls();
   updatePreview();
@@ -762,7 +780,7 @@ void DlgSettingsCoords::slotTime(const QString &)
 {
   LOG4CPP_INFO_S ((*mainCat)) << "DlgSettingsCoords::slotTime";
 
-  CoordUnitsTime coordUnits = (CoordUnitsTime) m_cmbTime->currentData ().toInt();
+  CoordUnitsTime coordUnits = static_cast<CoordUnitsTime> (m_cmbTime->currentData ().toInt());
   m_modelCoordsAfter->setCoordUnitsTime(coordUnits);
   updateControls();
   updatePreview();
@@ -773,10 +791,10 @@ void DlgSettingsCoords::slotUnitsXTheta(const QString &)
   LOG4CPP_INFO_S ((*mainCat)) << "DlgSettingsCoords::slotUnitsXTheta";
 
   if (m_modelCoordsAfter->coordsType() == COORDS_TYPE_CARTESIAN) {
-    CoordUnitsNonPolarTheta coordUnits = (CoordUnitsNonPolarTheta) m_cmbXThetaUnits->currentData ().toInt ();
+    CoordUnitsNonPolarTheta coordUnits = static_cast<CoordUnitsNonPolarTheta> (m_cmbXThetaUnits->currentData ().toInt ());
     m_modelCoordsAfter->setCoordUnitsX(coordUnits);
   } else {
-    CoordUnitsPolarTheta coordUnits = (CoordUnitsPolarTheta) m_cmbXThetaUnits->currentData ().toInt ();
+    CoordUnitsPolarTheta coordUnits = static_cast<CoordUnitsPolarTheta> (m_cmbXThetaUnits->currentData ().toInt ());
     m_modelCoordsAfter->setCoordUnitsTheta(coordUnits);
   }
   updateControls ();
@@ -787,7 +805,7 @@ void DlgSettingsCoords::slotUnitsYRadius(const QString &)
 {
   LOG4CPP_INFO_S ((*mainCat)) << "DlgSettingsCoords::slotUnitsYRadius";
 
-  CoordUnitsNonPolarTheta coordUnits = (CoordUnitsNonPolarTheta) m_cmbYRadiusUnits->currentData ().toInt ();
+  CoordUnitsNonPolarTheta coordUnits = static_cast<CoordUnitsNonPolarTheta> (m_cmbYRadiusUnits->currentData ().toInt ());
   if (m_modelCoordsAfter->coordsType() == COORDS_TYPE_CARTESIAN) {
     m_modelCoordsAfter->setCoordUnitsY(coordUnits);
   } else {
@@ -797,6 +815,11 @@ void DlgSettingsCoords::slotUnitsYRadius(const QString &)
   updatePreview();
 }
 
+void DlgSettingsCoords::slotWhatsThis ()
+{
+  QWhatsThis::enterWhatsThisMode();
+}
+
 void DlgSettingsCoords::slotXThetaLinear()
 {
   LOG4CPP_INFO_S ((*mainCat)) << "DlgSettingsCoords::slotXThetaLinear";
@@ -915,10 +938,10 @@ void DlgSettingsCoords::updateControls ()
 
   bool enableDateTime;
   if (m_btnCartesian->isChecked()) {
-    enableDateTime = (((CoordUnitsNonPolarTheta) m_cmbXThetaUnits->currentData ().toInt() == COORD_UNITS_NON_POLAR_THETA_DATE_TIME) ||
-                      ((CoordUnitsNonPolarTheta) m_cmbYRadiusUnits->currentData ().toInt() == COORD_UNITS_NON_POLAR_THETA_DATE_TIME));
+    enableDateTime = ((static_cast<CoordUnitsNonPolarTheta> (m_cmbXThetaUnits->currentData ().toInt()) == COORD_UNITS_NON_POLAR_THETA_DATE_TIME) ||
+                      (static_cast<CoordUnitsNonPolarTheta> (m_cmbYRadiusUnits->currentData ().toInt()) == COORD_UNITS_NON_POLAR_THETA_DATE_TIME));
   } else {
-    enableDateTime = ((CoordUnitsNonPolarTheta) m_cmbYRadiusUnits->currentData ().toInt() == COORD_UNITS_NON_POLAR_THETA_DATE_TIME);
+    enableDateTime = (static_cast<CoordUnitsNonPolarTheta> (m_cmbYRadiusUnits->currentData ().toInt()) == COORD_UNITS_NON_POLAR_THETA_DATE_TIME);
   }
   m_cmbDate->setEnabled (enableDateTime);
   m_cmbTime->setEnabled (enableDateTime);
