@@ -4,6 +4,7 @@
  * LICENSE or go to gnu.org/licenses for details. Distribution requires prior written permission.     *
  ******************************************************************************************************/
 
+#include "ButtonWhatsThis.h"
 #include "CmdMediator.h"
 #include "CmdSettingsCurveProperties.h"
 #include "ColorPalette.h"
@@ -26,12 +27,14 @@
 #include <QLabel>
 #include <QLineEdit>
 #include <QListWidget>
+#include <qmath.h>
 #include <QPen>
 #include <QPushButton>
 #include <QSettings>
 #include <QSpacerItem>
 #include <QSpinBox>
 #include <QTransform>
+#include <QWhatsThis>
 #include "Settings.h"
 #include "SettingsForGraph.h"
 #include "Spline.h"
@@ -48,6 +51,7 @@ const QString CONNECT_AS_RELATION_STRAIGHT_STR ("Relation - Straight");
 
 const double PREVIEW_WIDTH = 100.0;
 const double PREVIEW_HEIGHT = 100.0;
+const int MINIMUM_HEIGHT = 500;
 
 const QPointF POS_LEFT (PREVIEW_WIDTH / 3.0,
                         PREVIEW_HEIGHT * 2.0 / 3.0);
@@ -61,10 +65,10 @@ DlgSettingsCurveProperties::DlgSettingsCurveProperties(MainWindow &mainWindow) :
                            "DlgSettingsCurveProperties",
                            mainWindow),
   m_modelMainWindow (mainWindow.modelMainWindow()),
-  m_scenePreview (0),
-  m_viewPreview (0),
-  m_modelCurveStylesBefore (0),
-  m_modelCurveStylesAfter (0)
+  m_scenePreview (nullptr),
+  m_viewPreview (nullptr),
+  m_modelCurveStylesBefore (nullptr),
+  m_modelCurveStylesAfter (nullptr)
 {
   LOG4CPP_INFO_S ((*mainCat)) << "DlgSettingsCurveProperties::DlgSettingsCurveProperties";
 
@@ -84,13 +88,18 @@ void DlgSettingsCurveProperties::createCurveName (QGridLayout *layout,
 {
   LOG4CPP_INFO_S ((*mainCat)) << "DlgSettingsCurveProperties::createCurveName";
 
-  QLabel *labelCurveName = new QLabel (tr ("Curve Name:"));
-  layout->addWidget (labelCurveName, row, 1);
+  QLabel *labelCurveName = new QLabel (QString ("%1:").arg (tr ("Curve Name")));
+  layout->addWidget (labelCurveName, row, 1, 1, 1);
 
   m_cmbCurveName = new QComboBox ();
   m_cmbCurveName->setWhatsThis (tr ("Name of the curve that is currently selected for editing"));
   connect (m_cmbCurveName, SIGNAL (activated (const QString &)), this, SLOT (slotCurveName (const QString &))); // activated() ignores code changes
-  layout->addWidget (m_cmbCurveName, row++, 2);
+  layout->addWidget (m_cmbCurveName, row, 2, 1, 1);
+
+  createWhatsThis (layout,
+                   m_btnWhatsThis,
+                   row++,
+                   3);
 }
 
 void DlgSettingsCurveProperties::createLine (QGridLayout *layout,
@@ -104,7 +113,7 @@ void DlgSettingsCurveProperties::createLine (QGridLayout *layout,
   QGridLayout *layoutGroup = new QGridLayout;
   m_groupLine->setLayout (layoutGroup);
 
-  QLabel *labelLineWidth = new QLabel (tr ("Width:"));
+  QLabel *labelLineWidth = new QLabel (QString ("%1:").arg (tr ("Width")));
   layoutGroup->addWidget (labelLineWidth, 0, 0);
 
   m_spinLineWidth = new QSpinBox (m_groupLine);
@@ -114,7 +123,7 @@ void DlgSettingsCurveProperties::createLine (QGridLayout *layout,
   connect (m_spinLineWidth, SIGNAL (valueChanged (int)), this, SLOT (slotLineWidth (int)));
   layoutGroup->addWidget (m_spinLineWidth, 0, 1);
 
-  QLabel *labelLineColor = new QLabel (tr ("Color:"));
+  QLabel *labelLineColor = new QLabel (QString ("%1:").arg (tr ("Color")));
   layoutGroup->addWidget (labelLineColor, 1, 0);
 
   m_cmbLineColor = new QComboBox (m_groupLine);
@@ -124,7 +133,7 @@ void DlgSettingsCurveProperties::createLine (QGridLayout *layout,
   connect (m_cmbLineColor, SIGNAL (activated (const QString &)), this, SLOT (slotLineColor (const QString &))); // activated() ignores code changes
   layoutGroup->addWidget (m_cmbLineColor, 1, 1);
 
-  QLabel *labelLineType = new QLabel (tr ("Connect as:"));
+  QLabel *labelLineType = new QLabel (QString ("%1:").arg (tr ("Connect as")));
   layoutGroup->addWidget (labelLineType, 2, 0);
 
   m_cmbLineType = new QComboBox (m_groupLine);
@@ -141,7 +150,8 @@ void DlgSettingsCurveProperties::createLine (QGridLayout *layout,
                                    "endpoints.\n\n"
                                    "Lines are drawn between successively ordered points.\n\n"
                                    "Straight curves are drawn with straight lines between successive points. Smooth curves are drawn "
-                                   "with smooth lines between successive points.\n\n"
+                                   "with smooth lines between successive points, using natural cubic splines of (x,y) pairs versus "
+                                   "scalar ordinal (t) values.\n\n"
                                     "This applies only to graph curves. No lines are ever drawn between axis points."));
   connect (m_cmbLineType, SIGNAL (activated (const QString &)), this, SLOT (slotLineType (const QString &))); // activated() ignores code changes
   layoutGroup->addWidget (m_cmbLineType, 2, 1);
@@ -158,7 +168,7 @@ void DlgSettingsCurveProperties::createPoint (QGridLayout *layout,
   QGridLayout *layoutGroup = new QGridLayout;
   m_groupPoint->setLayout (layoutGroup);
 
-  QLabel *labelPointShape = new QLabel(tr ("Shape:"));
+  QLabel *labelPointShape = new QLabel(QString ("%1:").arg (tr ("Shape")));
   layoutGroup->addWidget (labelPointShape, 0, 0);
 
   m_cmbPointShape = new QComboBox (m_groupPoint);
@@ -169,16 +179,20 @@ void DlgSettingsCurveProperties::createPoint (QGridLayout *layout,
                             POINT_SHAPE_CROSS);
   m_cmbPointShape->addItem (pointShapeToString (POINT_SHAPE_DIAMOND),
                             POINT_SHAPE_DIAMOND);
+  m_cmbPointShape->addItem (pointShapeToString (POINT_SHAPE_HOURGLASS),
+                            POINT_SHAPE_HOURGLASS);  
   m_cmbPointShape->addItem (pointShapeToString (POINT_SHAPE_SQUARE),
                             POINT_SHAPE_SQUARE);
   m_cmbPointShape->addItem (pointShapeToString (POINT_SHAPE_TRIANGLE),
                             POINT_SHAPE_TRIANGLE);
+  m_cmbPointShape->addItem (pointShapeToString (POINT_SHAPE_TRIANGLE2),
+                            POINT_SHAPE_TRIANGLE2);
   m_cmbPointShape->addItem (pointShapeToString (POINT_SHAPE_X),
                             POINT_SHAPE_X);
   connect (m_cmbPointShape, SIGNAL (activated (const QString &)), this, SLOT (slotPointShape (const QString &))); // activated() ignores code changes
   layoutGroup->addWidget (m_cmbPointShape, 0, 1);
 
-  QLabel *labelPointRadius = new QLabel (tr ("Radius:"));
+  QLabel *labelPointRadius = new QLabel (QString ("%1:").arg (tr ("Radius")));
   layoutGroup->addWidget (labelPointRadius, 1, 0);
 
   m_spinPointRadius = new QSpinBox (m_groupPoint);
@@ -187,7 +201,7 @@ void DlgSettingsCurveProperties::createPoint (QGridLayout *layout,
   connect (m_spinPointRadius, SIGNAL (valueChanged (int)), this, SLOT (slotPointRadius (int)));
   layoutGroup->addWidget (m_spinPointRadius, 1, 1);
 
-  QLabel *labelPointLineWidth = new QLabel (tr ("Line width:"));
+  QLabel *labelPointLineWidth = new QLabel (QString ("%1:").arg (tr ("Line width")));
   layoutGroup->addWidget (labelPointLineWidth, 2, 0);
 
   m_spinPointLineWidth = new QSpinBox (m_groupPoint);
@@ -199,7 +213,7 @@ void DlgSettingsCurveProperties::createPoint (QGridLayout *layout,
   connect (m_spinPointLineWidth, SIGNAL (valueChanged (int)), this, SLOT (slotPointLineWidth (int)));
   layoutGroup->addWidget (m_spinPointLineWidth, 2, 1);
 
-  QLabel *labelPointColor = new QLabel (tr ("Color:"));
+  QLabel *labelPointColor = new QLabel (QString ("%1:").arg (tr ("Color")));
   layoutGroup->addWidget (labelPointColor, 3, 0);
 
   m_cmbPointColor = new QComboBox (m_groupPoint);
@@ -328,7 +342,7 @@ void DlgSettingsCurveProperties::drawLine (bool isRelation,
 void DlgSettingsCurveProperties::drawPoints (const PointStyle &pointStyle)
 {
   const QString NULL_IDENTIFIER;
-  GeometryWindow *NULL_GEOMETRY_WINDOW = 0;
+  GeometryWindow *NULL_GEOMETRY_WINDOW = nullptr;
 
   GraphicsPointFactory pointFactory;
 
@@ -380,12 +394,8 @@ void DlgSettingsCurveProperties::load (CmdMediator &cmdMediator)
   setCmdMediator (cmdMediator);
 
   // Flush old data
-  if (m_modelCurveStylesBefore != 0) {
-    delete m_modelCurveStylesBefore;
-  }
-  if (m_modelCurveStylesAfter != 0) {
-    delete m_modelCurveStylesAfter;
-  }
+  delete m_modelCurveStylesBefore;
+  delete m_modelCurveStylesAfter;
 
   // Save new data
   m_modelCurveStylesBefore = new CurveStyles (cmdMediator.coordSystem());
@@ -449,10 +459,10 @@ void DlgSettingsCurveProperties::loadForCurveName (const QString &curveName)
 void DlgSettingsCurveProperties::resetSceneRectangle ()
 {
 
-  QRect rect (0.0,
-              0.0,
-              PREVIEW_WIDTH,
-              PREVIEW_HEIGHT);
+  QRect rect (0,
+              0,
+              qFloor (PREVIEW_WIDTH),
+              qFloor (PREVIEW_HEIGHT));
 
   QGraphicsRectItem *itemPerimeter = new QGraphicsRectItem(rect);
   itemPerimeter->setVisible(false);
@@ -466,6 +476,13 @@ void DlgSettingsCurveProperties::setCurveName (const QString &curveName)
   loadForCurveName (curveName);
 }
 
+void DlgSettingsCurveProperties::setSmallDialogs(bool smallDialogs)
+{
+  if (!smallDialogs) {
+    setMinimumHeight (MINIMUM_HEIGHT);
+  }
+}
+
 void DlgSettingsCurveProperties::slotCurveName(const QString &curveName)
 {
   LOG4CPP_INFO_S ((*mainCat)) << "DlgSettingsCurveProperties::slotCurveName";
@@ -473,7 +490,7 @@ void DlgSettingsCurveProperties::slotCurveName(const QString &curveName)
   // Dirty flag is not set when simply changing to new curve
 
   // Do nothing if combobox is getting cleared, or load has not been called yet
-  if (!curveName.isEmpty () && (m_modelCurveStylesAfter != 0)) {
+  if (!curveName.isEmpty () && (m_modelCurveStylesAfter != nullptr)) {
 
     loadForCurveName (curveName);
   }
@@ -486,7 +503,7 @@ void DlgSettingsCurveProperties::slotLineColor(const QString &lineColor)
   m_isDirty = true;
 
   m_modelCurveStylesAfter->setLineColor(m_cmbCurveName->currentText(),
-                                        (ColorPalette) m_cmbLineColor->currentData().toInt());
+                                        static_cast<ColorPalette> (m_cmbLineColor->currentData().toInt()));
   updateControls();
   updatePreview();
 }
@@ -510,7 +527,7 @@ void DlgSettingsCurveProperties::slotLineType(const QString &lineType)
   m_isDirty = true;
 
   m_modelCurveStylesAfter->setLineConnectAs(m_cmbCurveName->currentText(),
-                                            (CurveConnectAs) m_cmbLineType->currentData().toInt ());
+                                            static_cast<CurveConnectAs> (m_cmbLineType->currentData().toInt ()));
   updateControls();
   updatePreview();
 }
@@ -522,7 +539,7 @@ void DlgSettingsCurveProperties::slotPointColor(const QString &pointColor)
   m_isDirty = true;
 
   m_modelCurveStylesAfter->setPointColor(m_cmbCurveName->currentText(),
-                                         (ColorPalette) m_cmbPointColor->currentData().toInt ());
+                                         static_cast<ColorPalette> (m_cmbPointColor->currentData().toInt ()));
   updateControls();
   updatePreview();
 }
@@ -558,7 +575,7 @@ void DlgSettingsCurveProperties::slotPointShape(const QString &)
   m_isDirty = true;
 
   m_modelCurveStylesAfter->setPointShape(m_cmbCurveName->currentText(),
-                                         (PointShape) m_cmbPointShape->currentData().toInt ());
+                                         static_cast<PointShape> (m_cmbPointShape->currentData().toInt ()));
   updateControls();
   updatePreview();
 }
@@ -599,6 +616,11 @@ void DlgSettingsCurveProperties::slotSaveDefault()
   settings.endGroup ();
 }
 
+void DlgSettingsCurveProperties::slotWhatsThis ()
+{
+  QWhatsThis::enterWhatsThisMode();
+}
+
 void DlgSettingsCurveProperties::updateControls()
 {
   bool isGoodState = !m_spinPointRadius->text().isEmpty () &&
