@@ -4,10 +4,13 @@
  * LICENSE or go to gnu.org/licenses for details. Distribution requires prior written permission.     *
  ******************************************************************************************************/
 
+#include "ButtonWhatsThis.h"
 #include "CmdMediator.h"
 #include "CmdSettingsGridRemoval.h"
 #include "DlgSettingsGridRemoval.h"
 #include "EngaugeAssert.h"
+#include "GridInitializer.h"
+#include "GridRemoval.h"
 #include "Logger.h"
 #include "MainWindow.h"
 #include <QCheckBox>
@@ -19,6 +22,8 @@
 #include <QHBoxLayout>
 #include <QLabel>
 #include <QLineEdit>
+#include <QPixmap>
+#include <QWhatsThis>
 #include "ViewPreview.h"
 
 const double CLOSE_DISTANCE_MAX = 64;
@@ -27,15 +32,25 @@ const int CLOSE_DECIMALS = 1;
 const int COUNT_MIN = 1;
 const int COUNT_MAX = 100;
 const int COUNT_DECIMALS = 0;
+const int MINIMUM_HEIGHT = 520;
 
 DlgSettingsGridRemoval::DlgSettingsGridRemoval(MainWindow &mainWindow) :
   DlgSettingsAbstractBase (tr ("Grid Removal"),
                            "DlgSettingsGridRemoval",
                            mainWindow),
-  m_scenePreview (0),
-  m_viewPreview (0),
-  m_modelGridRemovalBefore (0),
-  m_modelGridRemovalAfter (0)
+  m_validatorCloseDistance (nullptr),
+  m_validatorCountX (nullptr),
+  m_validatorStartX (nullptr),
+  m_validatorStepX (nullptr),
+  m_validatorStopX (nullptr),      
+  m_validatorCountY (nullptr),
+  m_validatorStartY (nullptr),
+  m_validatorStepY (nullptr),
+  m_validatorStopY (nullptr),  
+  m_scenePreview (nullptr),
+  m_viewPreview (nullptr),
+  m_modelGridRemovalBefore (nullptr),
+  m_modelGridRemovalAfter (nullptr)
 {
   LOG4CPP_INFO_S ((*mainCat)) << "DlgSettingsGridRemoval::DlgSettingsGridRemoval";
 
@@ -46,6 +61,16 @@ DlgSettingsGridRemoval::DlgSettingsGridRemoval(MainWindow &mainWindow) :
 DlgSettingsGridRemoval::~DlgSettingsGridRemoval()
 {
   LOG4CPP_INFO_S ((*mainCat)) << "DlgSettingsGridRemoval::~DlgSettingsGridRemoval";
+
+  delete m_validatorCloseDistance;
+  delete m_validatorCountX;
+  delete m_validatorStartX;
+  delete m_validatorStepX;
+  delete m_validatorStopX;      
+  delete m_validatorCountY;
+  delete m_validatorStartY;
+  delete m_validatorStepY;
+  delete m_validatorStopY;
 }
 
 void DlgSettingsGridRemoval::createOptionalSaveDefault (QHBoxLayout * /* layout */)
@@ -80,7 +105,7 @@ void DlgSettingsGridRemoval::createRemoveGridLines (QGridLayout *layout, int &ro
   connect (m_chkRemoveGridLines, SIGNAL (stateChanged (int)), this, SLOT (slotRemoveGridLines (int)));
   layout->addWidget (m_chkRemoveGridLines, row++, 1, 1, 3);
 
-  QLabel *labelCloseDistance = new QLabel (tr ("Close distance (pixels):"));
+  QLabel *labelCloseDistance = new QLabel (QString ("%1:").arg (tr ("Close distance (pixels)")));
   layout->addWidget (labelCloseDistance, row, 2);
 
   m_editCloseDistance = new QLineEdit;
@@ -111,7 +136,7 @@ void DlgSettingsGridRemoval::createRemoveGridLinesX (QGridLayout *layout, int &r
   QGridLayout *layoutGroup = new QGridLayout;
   groupX->setLayout (layoutGroup);
 
-  QLabel *labelDisable = new QLabel (tr ("Disable:"));
+  QLabel *labelDisable = new QLabel (QString ("%1:").arg (tr ("Disable")));
   layoutGroup->addWidget (labelDisable, 0, 0);
 
   m_cmbDisableX = new QComboBox;
@@ -130,7 +155,7 @@ void DlgSettingsGridRemoval::createRemoveGridLinesX (QGridLayout *layout, int &r
   connect (m_cmbDisableX, SIGNAL (activated (const QString &)), this, SLOT (slotDisableX (const QString &))); // activated() ignores code changes
   layoutGroup->addWidget (m_cmbDisableX, 0, 1);
 
-  QLabel *labelCount = new QLabel (tr ("Count:"));
+  QLabel *labelCount = new QLabel (QString ("%1:").arg (tr ("Count")));
   layoutGroup->addWidget (labelCount, 1, 0);
 
   m_editCountX = new QLineEdit;
@@ -141,7 +166,7 @@ void DlgSettingsGridRemoval::createRemoveGridLinesX (QGridLayout *layout, int &r
   connect (m_editCountX, SIGNAL (textChanged (const QString &)), this, SLOT  (slotCountX (const QString &)));
   layoutGroup->addWidget (m_editCountX, 1, 1);
 
-  QLabel *labelStart = new QLabel (tr ("Start:"));
+  QLabel *labelStart = new QLabel (QString ("%1:").arg (tr ("Start")));
   layoutGroup->addWidget (labelStart, 2, 0);
 
   m_editStartX = new QLineEdit;
@@ -152,18 +177,18 @@ void DlgSettingsGridRemoval::createRemoveGridLinesX (QGridLayout *layout, int &r
   connect (m_editStartX, SIGNAL (textChanged (const QString &)), this, SLOT  (slotStartX (const QString &)));
   layoutGroup->addWidget (m_editStartX, 2, 1);
 
-  QLabel *labelStep = new QLabel (tr ("Step:"));
+  QLabel *labelStep = new QLabel (QString ("%1:").arg (tr ("Step")));
   layoutGroup->addWidget (labelStep, 3, 0);
 
   m_editStepX = new QLineEdit;
   m_editStepX->setWhatsThis (tr ("Difference in value between two successive X grid lines.\n\n"
-                                 "The step value must be greater than zero"));
+                                 "The step value must be greater than zero (linear) or one (log)"));
   m_validatorStepX = new QDoubleValidator;
   m_editStepX->setValidator (m_validatorStepX);
   connect (m_editStepX, SIGNAL (textChanged (const QString &)), this, SLOT  (slotStepX (const QString &)));
   layoutGroup->addWidget (m_editStepX, 3, 1);
 
-  QLabel *labelStop = new QLabel (tr ("Stop:"));
+  QLabel *labelStop = new QLabel (QString ("%1:").arg (tr ("Stop")));
   layoutGroup->addWidget (labelStop, 4, 0);
 
   m_editStopX = new QLineEdit;
@@ -189,7 +214,7 @@ void DlgSettingsGridRemoval::createRemoveGridLinesY (QGridLayout *layout, int &r
   QGridLayout *layoutGroup = new QGridLayout;
   groupY->setLayout (layoutGroup);
 
-  QLabel *labelDisable = new QLabel (tr ("Disable:"));
+  QLabel *labelDisable = new QLabel (QString ("%1:").arg (tr ("Disable")));
   layoutGroup->addWidget (labelDisable, 0, 0);
 
   m_cmbDisableY = new QComboBox;
@@ -208,7 +233,7 @@ void DlgSettingsGridRemoval::createRemoveGridLinesY (QGridLayout *layout, int &r
   connect (m_cmbDisableY, SIGNAL (activated (const QString &)), this, SLOT (slotDisableY (const QString &))); // activated() ignores code changes
   layoutGroup->addWidget (m_cmbDisableY, 0, 1);
 
-  QLabel *labelCount = new QLabel (tr ("Count:"));
+  QLabel *labelCount = new QLabel (QString ("%1:").arg (tr ("Count")));
   layoutGroup->addWidget (labelCount, 1, 0);
 
   m_editCountY = new QLineEdit;
@@ -219,7 +244,7 @@ void DlgSettingsGridRemoval::createRemoveGridLinesY (QGridLayout *layout, int &r
   connect (m_editCountY, SIGNAL (textChanged (const QString &)), this, SLOT  (slotCountY (const QString &)));
   layoutGroup->addWidget (m_editCountY, 1, 1);
 
-  QLabel *labelStart = new QLabel (tr ("Start:"));
+  QLabel *labelStart = new QLabel (QString ("%1:").arg (tr ("Start")));
   layoutGroup->addWidget (labelStart, 2, 0);
 
   m_editStartY = new QLineEdit;
@@ -230,18 +255,18 @@ void DlgSettingsGridRemoval::createRemoveGridLinesY (QGridLayout *layout, int &r
   connect (m_editStartY, SIGNAL (textChanged (const QString &)), this, SLOT  (slotStartY (const QString &)));
   layoutGroup->addWidget (m_editStartY, 2, 1);
 
-  QLabel *labelStep = new QLabel (tr ("Step:"));
+  QLabel *labelStep = new QLabel (QString ("%1:").arg (tr ("Step")));
   layoutGroup->addWidget (labelStep, 3, 0);
 
   m_editStepY = new QLineEdit;
   m_editStepY->setWhatsThis (tr ("Difference in value between two successive Y grid lines.\n\n"
-                                 "The step value must be greater than zero"));
+                                 "The step value must be greater than zero (linear) or one (log)"));
   m_validatorStepY = new QDoubleValidator;
   m_editStepY->setValidator (m_validatorStepY);
   connect (m_editStepY, SIGNAL (textChanged (const QString &)), this, SLOT  (slotStepY (const QString &)));
   layoutGroup->addWidget (m_editStepY, 3, 1);
 
-  QLabel *labelStop = new QLabel (tr ("Stop:"));
+  QLabel *labelStop = new QLabel (QString ("%1:").arg (tr ("Stop")));
   layoutGroup->addWidget (labelStop, 4, 0);
 
   m_editStopY = new QLineEdit;
@@ -257,20 +282,23 @@ QWidget *DlgSettingsGridRemoval::createSubPanel ()
 {
   LOG4CPP_INFO_S ((*mainCat)) << "DlgSettingsGridRemoval::createSubPanel";
 
-  const int COLUMN_CHECKBOX_WIDTH = 60;
-
   QWidget *subPanel = new QWidget ();
   QGridLayout *layout = new QGridLayout (subPanel);
   subPanel->setLayout (layout);
 
   layout->setColumnStretch(0, 1); // Empty first column
   layout->setColumnStretch(1, 0); // Checkbox part of "section" checkboxes. In other rows this has empty space as indentation
-  layout->setColumnMinimumWidth(1, COLUMN_CHECKBOX_WIDTH);
   layout->setColumnStretch(2, 0); // X
   layout->setColumnStretch(3, 0); // Y
   layout->setColumnStretch(4, 1); // Empty last column
 
   int row = 0;
+
+  createWhatsThis (layout,
+                   m_btnWhatsThis,
+                   row++,
+                   4);
+
   createRemoveGridLines (layout, row);
   createPreview (layout, row);
 
@@ -300,12 +328,8 @@ void DlgSettingsGridRemoval::load (CmdMediator &cmdMediator)
   setCmdMediator (cmdMediator);
 
   // Flush old data
-  if (m_modelGridRemovalBefore != 0) {
-    delete m_modelGridRemovalBefore;
-  }
-  if (m_modelGridRemovalAfter != 0) {
-    delete m_modelGridRemovalAfter;
-  }
+  delete m_modelGridRemovalBefore;
+  delete m_modelGridRemovalAfter;
 
   // Save new data
   m_modelGridRemovalBefore = new DocumentModelGridRemoval (cmdMediator.document());
@@ -336,14 +360,18 @@ void DlgSettingsGridRemoval::load (CmdMediator &cmdMediator)
   m_editStepY->setText(QString::number(m_modelGridRemovalAfter->stepY()));
   m_editStopY->setText(QString::number(m_modelGridRemovalAfter->stopY()));
 
-  m_scenePreview->clear();
-  m_scenePreview->addPixmap (cmdMediator.document().pixmap());
-
   updateControls ();
   enableOk (false); // Disable Ok button since there not yet any changes
   updatePreview();
 }
 
+void DlgSettingsGridRemoval::setSmallDialogs(bool smallDialogs)
+{
+  if (!smallDialogs) {
+    setMinimumHeight (MINIMUM_HEIGHT);
+  }
+}
+
 void DlgSettingsGridRemoval::slotCloseDistance(const QString &)
 {
   LOG4CPP_INFO_S ((*mainCat)) << "DlgSettingsGridRemoval::slotCloseDistance";
@@ -358,6 +386,7 @@ void DlgSettingsGridRemoval::slotCountX(const QString &count)
   LOG4CPP_INFO_S ((*mainCat)) << "DlgSettingsGridRemoval::slotCountX";
 
   m_modelGridRemovalAfter->setCountX(count.toInt());
+  updateDisplayedVariableX ();
   updateControls ();
   updatePreview();
 }
@@ -367,6 +396,7 @@ void DlgSettingsGridRemoval::slotCountY(const QString &count)
   LOG4CPP_INFO_S ((*mainCat)) << "DlgSettingsGridRemoval::slotCountY";
 
   m_modelGridRemovalAfter->setCountY(count.toInt());
+  updateDisplayedVariableY ();
   updateControls ();
   updatePreview();
 }
@@ -375,8 +405,9 @@ void DlgSettingsGridRemoval::slotDisableX(const QString &)
 {
   LOG4CPP_INFO_S ((*mainCat)) << "DlgSettingsGridRemoval::slotDisableX";
 
-  GridCoordDisable gridCoordDisable = (GridCoordDisable) m_cmbDisableX->currentData().toInt();
+  GridCoordDisable gridCoordDisable = static_cast<GridCoordDisable> (m_cmbDisableX->currentData().toInt());
   m_modelGridRemovalAfter->setGridCoordDisableX(gridCoordDisable);
+  updateDisplayedVariableX ();
   updateControls();
   updatePreview();
 }
@@ -385,8 +416,9 @@ void DlgSettingsGridRemoval::slotDisableY(const QString &)
 {
   LOG4CPP_INFO_S ((*mainCat)) << "DlgSettingsGridRemoval::slotDisableY";
 
-  GridCoordDisable gridCoordDisable = (GridCoordDisable) m_cmbDisableY->currentData().toInt();
+  GridCoordDisable gridCoordDisable = static_cast<GridCoordDisable> (m_cmbDisableY->currentData().toInt());
   m_modelGridRemovalAfter->setGridCoordDisableY(gridCoordDisable);
+  updateDisplayedVariableY ();
   updateControls();
   updatePreview();
 }
@@ -405,6 +437,7 @@ void DlgSettingsGridRemoval::slotStartX(const QString &startX)
   LOG4CPP_INFO_S ((*mainCat)) << "DlgSettingsGridRemoval::slotStartX";
 
   m_modelGridRemovalAfter->setStartX(startX.toDouble());
+  updateDisplayedVariableX ();
   updateControls();
   updatePreview();
 }
@@ -414,6 +447,7 @@ void DlgSettingsGridRemoval::slotStartY(const QString &startY)
   LOG4CPP_INFO_S ((*mainCat)) << "DlgSettingsGridRemoval::slotStartY";
 
   m_modelGridRemovalAfter->setStartY(startY.toDouble());
+  updateDisplayedVariableY ();
   updateControls();
   updatePreview();
 }
@@ -423,6 +457,7 @@ void DlgSettingsGridRemoval::slotStepX(const QString &stepX)
   LOG4CPP_INFO_S ((*mainCat)) << "DlgSettingsGridRemoval::slotStepX";
 
   m_modelGridRemovalAfter->setStepX(stepX.toDouble());
+  updateDisplayedVariableX ();
   updateControls();
   updatePreview();
 }
@@ -432,6 +467,7 @@ void DlgSettingsGridRemoval::slotStepY(const QString &stepY)
   LOG4CPP_INFO_S ((*mainCat)) << "DlgSettingsGridRemoval::slotStepY";
 
   m_modelGridRemovalAfter->setStepY(stepY.toDouble());
+  updateDisplayedVariableY ();
   updateControls();
   updatePreview();
 }
@@ -441,6 +477,7 @@ void DlgSettingsGridRemoval::slotStopX(const QString &stopX)
   LOG4CPP_INFO_S ((*mainCat)) << "DlgSettingsGridRemoval::slotStopX";
 
   m_modelGridRemovalAfter->setStopX(stopX.toDouble());
+  updateDisplayedVariableX ();
   updateControls();
   updatePreview();
 }
@@ -450,17 +487,23 @@ void DlgSettingsGridRemoval::slotStopY(const QString &stopY)
   LOG4CPP_INFO_S ((*mainCat)) << "DlgSettingsGridRemoval::slotStopY";
 
   m_modelGridRemovalAfter->setStopY(stopY.toDouble());
+  updateDisplayedVariableY ();
   updateControls();
   updatePreview();
 }
 
+void DlgSettingsGridRemoval::slotWhatsThis ()
+{
+  QWhatsThis::enterWhatsThisMode();
+}
+
 void DlgSettingsGridRemoval::updateControls ()
 {
   m_editCloseDistance->setEnabled (m_chkRemoveGridLines->isChecked ());
 
   m_cmbDisableX->setEnabled (m_chkRemoveGridLines->isChecked ());
 
-  GridCoordDisable disableX = (GridCoordDisable) m_cmbDisableX->currentData().toInt();
+  GridCoordDisable disableX = static_cast<GridCoordDisable> (m_cmbDisableX->currentData().toInt());
   m_editCountX->setEnabled (m_chkRemoveGridLines->isChecked () && (disableX != GRID_COORD_DISABLE_COUNT));
   m_editStartX->setEnabled (m_chkRemoveGridLines->isChecked () && (disableX != GRID_COORD_DISABLE_START));
   m_editStepX->setEnabled (m_chkRemoveGridLines->isChecked () && (disableX != GRID_COORD_DISABLE_STEP));
@@ -468,7 +511,7 @@ void DlgSettingsGridRemoval::updateControls ()
 
   m_cmbDisableY->setEnabled (m_chkRemoveGridLines->isChecked ());
 
-  GridCoordDisable disableY = (GridCoordDisable) m_cmbDisableY->currentData().toInt();
+  GridCoordDisable disableY = static_cast<GridCoordDisable> (m_cmbDisableY->currentData().toInt());
   m_editCountY->setEnabled (m_chkRemoveGridLines->isChecked () && (disableY != GRID_COORD_DISABLE_COUNT));
   m_editStartY->setEnabled (m_chkRemoveGridLines->isChecked () && (disableY != GRID_COORD_DISABLE_START));
   m_editStepY->setEnabled (m_chkRemoveGridLines->isChecked () && (disableY != GRID_COORD_DISABLE_STEP));
@@ -497,7 +540,89 @@ void DlgSettingsGridRemoval::updateControls ()
   enableOk (isOk);
 }
 
+void DlgSettingsGridRemoval::updateDisplayedVariableX ()
+{
+  GridInitializer initializer;
+
+  bool linearAxis = (cmdMediator ().document ().modelCoords ().coordScaleXTheta() == COORD_SCALE_LINEAR);
+
+  switch (m_modelGridRemovalAfter->gridCoordDisableX()) {
+    case GRID_COORD_DISABLE_COUNT:
+      m_editCountX->setText (QString::number (initializer.computeCount (linearAxis,
+                                                                        m_modelGridRemovalAfter->startX (),
+                                                                        m_modelGridRemovalAfter->stopX (),
+                                                                        m_modelGridRemovalAfter->stepX ())));
+      break;
+
+    case GRID_COORD_DISABLE_START:
+      m_editStartX->setText (QString::number (initializer.computeStart (linearAxis,
+                                                                        m_modelGridRemovalAfter->stopX (),
+                                                                        m_modelGridRemovalAfter->stepX (),
+                                                                        m_modelGridRemovalAfter->countX ())));
+      break;
+
+    case GRID_COORD_DISABLE_STEP:
+      m_editStepX->setText (QString::number (initializer.computeStep (linearAxis,
+                                                                      m_modelGridRemovalAfter->startX (),
+                                                                      m_modelGridRemovalAfter->stopX (),
+                                                                      m_modelGridRemovalAfter->countX ())));
+      break;
+
+    case GRID_COORD_DISABLE_STOP:
+      m_editStopX->setText (QString::number (initializer.computeStop (linearAxis,
+                                                                      m_modelGridRemovalAfter->startX (),
+                                                                      m_modelGridRemovalAfter->stepX (),
+                                                                      m_modelGridRemovalAfter->countX ())));
+      break;
+  }
+}
+
+void DlgSettingsGridRemoval::updateDisplayedVariableY ()
+{
+  GridInitializer initializer;
+
+  bool linearAxis = (cmdMediator ().document ().modelCoords ().coordScaleYRadius () == COORD_SCALE_LINEAR);
+
+  switch (m_modelGridRemovalAfter->gridCoordDisableY()) {
+    case GRID_COORD_DISABLE_COUNT:
+      m_editCountY->setText (QString::number (initializer.computeCount (linearAxis,
+                                                                        m_modelGridRemovalAfter->startY (),
+                                                                        m_modelGridRemovalAfter->stopY (),
+                                                                        m_modelGridRemovalAfter->stepY ())));
+      break;
+
+    case GRID_COORD_DISABLE_START:
+      m_editStartY->setText (QString::number (initializer.computeStart (linearAxis,
+                                                                        m_modelGridRemovalAfter->stopY (),
+                                                                        m_modelGridRemovalAfter->stepY (),
+                                                                        m_modelGridRemovalAfter->countY ())));
+      break;
+
+    case GRID_COORD_DISABLE_STEP:
+      m_editStepY->setText (QString::number (initializer.computeStep (linearAxis,
+                                                                      m_modelGridRemovalAfter->startY (),
+                                                                      m_modelGridRemovalAfter->stopY (),
+                                                                      m_modelGridRemovalAfter->countY ())));
+      break;
+
+    case GRID_COORD_DISABLE_STOP:
+      m_editStopY->setText (QString::number (initializer.computeStop (linearAxis,
+                                                                      m_modelGridRemovalAfter->startY (),
+                                                                      m_modelGridRemovalAfter->stepY (),
+                                                                      m_modelGridRemovalAfter->countY ())));
+      break;
+  }
+}
+
 void DlgSettingsGridRemoval::updatePreview ()
 {
+  GridRemoval gridRemoval (mainWindow().isGnuplot());
 
+  QPixmap pixmap = gridRemoval.remove (mainWindow ().transformation(),
+                                       *m_modelGridRemovalAfter,
+                                       cmdMediator ().document().pixmap().toImage());
+
+  m_scenePreview->clear();
+  addPixmap (*m_scenePreview,
+             pixmap);
 }
