@@ -4,11 +4,13 @@
  * LICENSE or go to gnu.org/licenses for details. Distribution requires prior written permission.     *
  ******************************************************************************************************/
 
+#include "ButtonWhatsThis.h"
 #include "CmdMediator.h"
 #include "CmdSettingsSegments.h"
 #include "DlgSettingsSegments.h"
 #include "EngaugeAssert.h"
 #include "GeometryWindow.h"
+#include "InactiveOpacity.h"
 #include "Logger.h"
 #include "MainWindow.h"
 #include "PointStyle.h"
@@ -19,19 +21,19 @@
 #include <QLabel>
 #include <qmath.h>
 #include <QSpinBox>
+#include <QWhatsThis>
 #include "Segment.h"
 #include "SegmentFactory.h"
 #include "ViewPreview.h"
 
+const int MINIMUM_HEIGHT = 540;
 const int MIN_LENGTH_MIN = 1;
 const int MIN_LENGTH_MAX = 10000;
 const int POINT_SEPARATION_MIN = 5;
 const int POINT_SEPARATION_MAX = 10000;
 
-const int IMAGE_WIDTH = 400;
-const int IMAGE_HEIGHT = 300;
-
-const double TWOPI = 2.0 * 3.1415926535;
+const int IMAGE_WIDTH = 200;
+const int IMAGE_HEIGHT = 350;
 
 const double BRUSH_WIDTH = 2.0;
 
@@ -39,10 +41,12 @@ DlgSettingsSegments::DlgSettingsSegments(MainWindow &mainWindow) :
   DlgSettingsAbstractBase (tr ("Segment Fill"),
                            "DlgSettingsSegments",
                            mainWindow),
-  m_scenePreview (0),
-  m_viewPreview (0),
-  m_modelSegmentsBefore (0),
-  m_modelSegmentsAfter (0),
+  m_scenePreviewActive (nullptr),
+  m_scenePreviewInactive (nullptr),
+  m_viewPreviewActive (nullptr),  
+  m_viewPreviewInactive (nullptr),
+  m_modelSegmentsBefore (nullptr),
+  m_modelSegmentsAfter (nullptr),
   m_loading (false)
 {
   LOG4CPP_INFO_S ((*mainCat)) << "DlgSettingsSegments::DlgSettingsSegments";
@@ -56,17 +60,15 @@ DlgSettingsSegments::~DlgSettingsSegments()
   LOG4CPP_INFO_S ((*mainCat)) << "DlgSettingsSegments::~DlgSettingsSegments";
 }
 
-void DlgSettingsSegments::clearPoints ()
+void DlgSettingsSegments::clearPointsOneScene (GraphicsPoints &points)
 {
-  LOG4CPP_INFO_S ((*mainCat)) << "DlgSettingsSegments::clearPoints";
-
   QList<GraphicsPoint*>::iterator itrP;
-  for (itrP = m_points.begin(); itrP != m_points.end(); itrP++) {
+  for (itrP = points.begin(); itrP != points.end(); itrP++) {
     GraphicsPoint *point = *itrP;
     delete point;
   }
 
-  m_points.clear();
+  points.clear();
 }
 
 void DlgSettingsSegments::createControls (QGridLayout *layout,
@@ -74,7 +76,7 @@ void DlgSettingsSegments::createControls (QGridLayout *layout,
 {
   LOG4CPP_INFO_S ((*mainCat)) << "DlgSettingsSegments::createControls";
 
-  QLabel *labelMinLength = new QLabel(tr ("Minimum length (points):"));
+  QLabel *labelMinLength = new QLabel(QString ("%1:").arg (tr ("Minimum length (points)")));
   layout->addWidget(labelMinLength, row, 1);
 
   m_spinMinLength = new QSpinBox;
@@ -86,7 +88,7 @@ void DlgSettingsSegments::createControls (QGridLayout *layout,
   connect (m_spinMinLength, SIGNAL (valueChanged (const QString &)), this, SLOT (slotMinLength (const QString &)));
   layout->addWidget(m_spinMinLength, row++, 2);
 
-  QLabel *labelPointSeparation = new QLabel(tr ("Point separation (pixels):"));
+  QLabel *labelPointSeparation = new QLabel(QString ("%1:").arg (tr ("Point separation (pixels)")));
   layout->addWidget (labelPointSeparation, row, 1);
 
   m_spinPointSeparation = new QSpinBox;
@@ -99,7 +101,7 @@ void DlgSettingsSegments::createControls (QGridLayout *layout,
   connect (m_spinPointSeparation, SIGNAL (valueChanged (const QString &)), this, SLOT (slotPointSeparation (const QString &)));
   layout->addWidget (m_spinPointSeparation, row++, 2);
 
-  QLabel *labelFillCorners = new QLabel (tr ("Fill corners:"));
+  QLabel *labelFillCorners = new QLabel (QString ("%1:").arg (tr ("Fill corners")));
   layout->addWidget (labelFillCorners, row, 1);
 
   m_chkFillCorners = new QCheckBox;
@@ -110,23 +112,60 @@ void DlgSettingsSegments::createControls (QGridLayout *layout,
   connect (m_chkFillCorners, SIGNAL (stateChanged (int)), this, SLOT (slotFillCorners (int)));
   layout->addWidget (m_chkFillCorners, row++, 2);
 
-  QLabel *labelLineWidth = new QLabel(tr ("Line width:"));
-  layout->addWidget (labelLineWidth, row, 1);
+  QLabel *labelLineWidthActive = new QLabel(QString ("%1:").arg (tr ("Active line width")));
+  layout->addWidget (labelLineWidthActive, row, 1);
 
-  m_spinLineWidth = new QSpinBox;
-  m_spinLineWidth->setWhatsThis (tr ("Select a size for the lines drawn along a segment"));
-  m_spinLineWidth->setMinimum(1);
-  connect (m_spinLineWidth, SIGNAL (valueChanged (int)), this, SLOT (slotLineWidth (int)));
-  layout->addWidget (m_spinLineWidth, row++, 2);
+  m_spinLineWidthActive = new QSpinBox;
+  m_spinLineWidthActive->setWhatsThis (tr ("Select a size for the lines drawn along a segment when hovering over the segment"));
+  m_spinLineWidthActive->setMinimum(1);
+  connect (m_spinLineWidthActive, SIGNAL (valueChanged (int)), this, SLOT (slotLineWidthActive (int)));
+  layout->addWidget (m_spinLineWidthActive, row++, 2);
 
-  QLabel *labelLineColor = new QLabel(tr ("Line color:"));
+  QLabel *labelLineWidthInactive = new QLabel(QString ("%1:").arg (tr ("Inactive line width")));
+  layout->addWidget (labelLineWidthInactive, row, 1);
+
+  m_spinLineWidthInactive = new QSpinBox;
+  m_spinLineWidthInactive->setWhatsThis (tr ("Select a size for the lines drawn along a segment when not hovering over the segment"));
+  m_spinLineWidthInactive->setMinimum(1);
+  connect (m_spinLineWidthInactive, SIGNAL (valueChanged (int)), this, SLOT (slotLineWidthInactive (int)));
+  layout->addWidget (m_spinLineWidthInactive, row++, 2);  
+
+  QLabel *labelLineColor = new QLabel(QString ("%1:").arg (tr ("Line color")));
   layout->addWidget (labelLineColor, row, 1);
 
   m_cmbLineColor = new QComboBox;
   m_cmbLineColor->setWhatsThis (tr ("Select a color for the lines drawn along a segment"));
   populateColorComboWithTransparent (*m_cmbLineColor);
-  connect (m_cmbLineColor, SIGNAL (activated (const QString &)), this, SLOT (slotLineColor (const QString &))); // activated() ignores code changes
+  connect (m_cmbLineColor, SIGNAL (activated (const QString &)),
+           this, SLOT (slotLineColor (const QString &))); // activated() ignores code changes
   layout->addWidget (m_cmbLineColor, row++, 2);
+
+  QLabel *labelInactiveOpacity = new QLabel(QString ("%1:").arg (tr ("Inactive opacity")));
+  layout->addWidget (labelInactiveOpacity, row, 1);
+
+  m_cmbInactiveOpacity = new QComboBox;
+  m_cmbInactiveOpacity->setWhatsThis (tr ("Select an opacity for the inactive segments that are not under the cursor"));
+  m_cmbInactiveOpacity->addItem (inactiveOpacityEnumToQString (INACTIVE_OPACITY_0),
+                                QVariant (INACTIVE_OPACITY_0));
+  m_cmbInactiveOpacity->addItem (inactiveOpacityEnumToQString (INACTIVE_OPACITY_32),
+                                QVariant (INACTIVE_OPACITY_32));
+  m_cmbInactiveOpacity->addItem (inactiveOpacityEnumToQString (INACTIVE_OPACITY_64),
+                                QVariant (INACTIVE_OPACITY_64));
+  m_cmbInactiveOpacity->addItem (inactiveOpacityEnumToQString (INACTIVE_OPACITY_96),
+                                QVariant (INACTIVE_OPACITY_96));
+  m_cmbInactiveOpacity->addItem (inactiveOpacityEnumToQString (INACTIVE_OPACITY_128),
+                                QVariant (INACTIVE_OPACITY_128));
+  m_cmbInactiveOpacity->addItem (inactiveOpacityEnumToQString (INACTIVE_OPACITY_160),
+                                QVariant (INACTIVE_OPACITY_160));
+  m_cmbInactiveOpacity->addItem (inactiveOpacityEnumToQString (INACTIVE_OPACITY_192),
+                                QVariant (INACTIVE_OPACITY_192));
+  m_cmbInactiveOpacity->addItem (inactiveOpacityEnumToQString (INACTIVE_OPACITY_224),
+                                QVariant (INACTIVE_OPACITY_224));
+  m_cmbInactiveOpacity->addItem (inactiveOpacityEnumToQString (INACTIVE_OPACITY_256),
+                                QVariant (INACTIVE_OPACITY_256));
+  connect (m_cmbInactiveOpacity, SIGNAL (activated (const QString &)),
+           this, SLOT (slotInactiveOpacity (const QString &))); // activated() ignores code changes
+  layout->addWidget (m_cmbInactiveOpacity, row++, 2);
 }
 
 void DlgSettingsSegments::createOptionalSaveDefault (QHBoxLayout * /* layout */)
@@ -138,20 +177,35 @@ void DlgSettingsSegments::createPreview (QGridLayout *layout,
 {
   LOG4CPP_INFO_S ((*mainCat)) << "DlgSettingsSegments::createPreview";
 
-  QLabel *labelPreview = new QLabel (tr ("Preview"));
-  layout->addWidget (labelPreview, row++, 0, 1, 4);
+  QLabel *labelPreviewActive = new QLabel (tr ("Hover Preview"));
+  layout->addWidget (labelPreviewActive, row, 0, 1, 2);
 
-  m_scenePreview = new QGraphicsScene (this);
-  m_viewPreview = new ViewPreview (m_scenePreview,
-                                   ViewPreview::VIEW_ASPECT_RATIO_VARIABLE,
-                                   this);
-  m_viewPreview->setWhatsThis (tr ("Preview window shows the shortest line that can be segment filled, "
-                                   "and the effects of current settings on segments and points generated by segment fill"));
-  m_viewPreview->setHorizontalScrollBarPolicy(Qt::ScrollBarAlwaysOff);
-  m_viewPreview->setVerticalScrollBarPolicy(Qt::ScrollBarAlwaysOff);
-  m_viewPreview->setMinimumHeight (MINIMUM_PREVIEW_HEIGHT);
+  QLabel *labelPreviewInactive = new QLabel (tr ("Inactive Preview"));
+  layout->addWidget (labelPreviewInactive, row++, 2, 1, 2);
 
-  layout->addWidget (m_viewPreview, row++, 0, 1, 4);
+  m_scenePreviewActive = new QGraphicsScene (this);
+  m_viewPreviewActive = new ViewPreview (m_scenePreviewActive,
+                                         ViewPreview::VIEW_ASPECT_RATIO_VARIABLE,
+                                         this);
+  m_viewPreviewActive->setWhatsThis (tr ("For segments under the cursor, this preview window shows the shortest "
+                                         "line that can be segment filled, "
+                                         "and the effects of current settings on segments and points generated by segment fill"));
+  m_viewPreviewActive->setHorizontalScrollBarPolicy(Qt::ScrollBarAlwaysOff);
+  m_viewPreviewActive->setVerticalScrollBarPolicy(Qt::ScrollBarAlwaysOff);
+  m_viewPreviewActive->setMinimumHeight (MINIMUM_PREVIEW_HEIGHT);
+  layout->addWidget (m_viewPreviewActive, row, 0, 1, 2);
+
+  m_scenePreviewInactive = new QGraphicsScene (this);
+  m_viewPreviewInactive = new ViewPreview (m_scenePreviewInactive,
+                                           ViewPreview::VIEW_ASPECT_RATIO_VARIABLE,
+                                           this);
+  m_viewPreviewInactive->setWhatsThis (tr ("For segments not under the cursor, this preview window shows the shortest "
+                                           "line that can be segment filled, "
+                                           "and the effects of current settings on segments and points generated by segment fill"));
+  m_viewPreviewInactive->setHorizontalScrollBarPolicy(Qt::ScrollBarAlwaysOff);
+  m_viewPreviewInactive->setVerticalScrollBarPolicy(Qt::ScrollBarAlwaysOff);
+  m_viewPreviewInactive->setMinimumHeight (MINIMUM_PREVIEW_HEIGHT);
+  layout->addWidget (m_viewPreviewInactive, row++, 2, 1, 2);
 }
 
 QImage DlgSettingsSegments::createPreviewImage () const
@@ -169,15 +223,15 @@ QImage DlgSettingsSegments::createPreviewImage () const
   int margin = IMAGE_WIDTH / 15;
   int yCenter = IMAGE_HEIGHT / 2;
   int yHeight = IMAGE_HEIGHT / 4;
-  int x, y, xLast, yLast;
+  int x, y, xLast = 0, yLast = 0;
   bool isFirst;
 
   // Draw sinusoid
   isFirst = true;
   int xStart = margin, xEnd = IMAGE_WIDTH / 2 - margin;
   for (x = xStart; x < xEnd; x++) {
-    double s = (double) (x - xStart) / (double) (xEnd - xStart);
-    int y = yCenter - yHeight * qSin (TWOPI * s);
+    double s = double (x - xStart) / double (xEnd - xStart);
+    int y = qFloor (yCenter - yHeight * qSin (2.0 * M_PI * s));
 
     if (!isFirst) {
       painter.drawLine (xLast, yLast, x, y);
@@ -189,15 +243,16 @@ QImage DlgSettingsSegments::createPreviewImage () const
 
   // Draw triangular waveform that looks like sinusoid straightened up into line segments
   isFirst = true;
-  xStart = IMAGE_WIDTH / 2 + margin, xEnd = IMAGE_WIDTH - margin;
+  xStart = IMAGE_WIDTH / 2 + margin;
+  xEnd = IMAGE_WIDTH - margin;
   for (x = xStart; x < xEnd; x++) {
-    double s = (double) (x - xStart) / (double) (xEnd - xStart);
+    double s = double (x - xStart) / double (xEnd - xStart);
     if (s <= 0.25) {
-      y = yCenter - yHeight * (4.0 * s);
+      y = qFloor (yCenter - yHeight * (4.0 * s));
     } else if (s < 0.75) {
-      y = yCenter - yHeight * (1.0 - 4.0 * (s - 0.25));
+      y = qFloor (yCenter - yHeight * (1.0 - 4.0 * (s - 0.25)));
     } else {
-      y = yCenter + yHeight * (1.0 - 4 * (s - 0.75));
+      y = qFloor (yCenter + yHeight * (1.0 - 4 * (s - 0.75)));
     }
 
     if (!isFirst) {
@@ -225,10 +280,18 @@ QWidget *DlgSettingsSegments::createSubPanel ()
   layout->setColumnStretch (3, 1); // Empty last column
 
   int row = 0;
+  createWhatsThis (layout,
+                   m_btnWhatsThis,
+                   row++,
+                   3);
   createControls(layout, row);
   createPreview (layout, row);
   QPixmap pixmap = QPixmap::fromImage (createPreviewImage());
-  m_scenePreview->addPixmap (pixmap);
+  addPixmap (*m_scenePreviewActive,
+             pixmap);
+  pixmap = QPixmap::fromImage (createPreviewImage());
+  addPixmap (*m_scenePreviewInactive,
+             pixmap);
 
   return subPanel;
 }
@@ -256,12 +319,8 @@ void DlgSettingsSegments::load (CmdMediator &cmdMediator)
   setCmdMediator (cmdMediator);
 
   // Flush old data
-  if (m_modelSegmentsBefore != 0) {
-    delete m_modelSegmentsBefore;
-  }
-  if (m_modelSegmentsAfter != 0) {
-    delete m_modelSegmentsAfter;
-  }
+  delete m_modelSegmentsBefore;
+  delete m_modelSegmentsAfter;
 
   // Save new data
   m_modelSegmentsBefore = new DocumentModelSegments (cmdMediator.document());
@@ -274,15 +333,20 @@ void DlgSettingsSegments::load (CmdMediator &cmdMediator)
   ENGAUGE_ASSERT (POINT_SEPARATION_MAX >= m_modelSegmentsAfter->pointSeparation());
 
   // Populate controls
-  m_spinPointSeparation->setValue (m_modelSegmentsAfter->pointSeparation());
-  m_spinMinLength->setValue (m_modelSegmentsAfter->minLength());
+  m_spinPointSeparation->setValue (qFloor (m_modelSegmentsAfter->pointSeparation()));
+  m_spinMinLength->setValue (qFloor (m_modelSegmentsAfter->minLength()));
   m_chkFillCorners->setChecked (m_modelSegmentsAfter->fillCorners ());
-  m_spinLineWidth->setValue (m_modelSegmentsAfter->lineWidth());
+  m_spinLineWidthActive->setValue (qFloor (m_modelSegmentsAfter->lineWidthActive()));
+  m_spinLineWidthInactive->setValue (qFloor (m_modelSegmentsAfter->lineWidthInactive()));  
 
   int indexLineColor = m_cmbLineColor->findData(QVariant (m_modelSegmentsAfter->lineColor()));
   ENGAUGE_ASSERT (indexLineColor >= 0);
   m_cmbLineColor->setCurrentIndex(indexLineColor);
 
+  int indexInactiveOpacity = m_cmbInactiveOpacity->findData (QVariant (m_modelSegmentsAfter->inactiveOpacity()));
+  ENGAUGE_ASSERT (indexInactiveOpacity >= 0);
+  m_cmbInactiveOpacity->setCurrentIndex (indexInactiveOpacity);
+
   // Loading finishes here
   m_loading = false;
 
@@ -291,6 +355,13 @@ void DlgSettingsSegments::load (CmdMediator &cmdMediator)
   updatePreview();
 }
 
+void DlgSettingsSegments::setSmallDialogs(bool smallDialogs)
+{
+  if (!smallDialogs) {
+    setMinimumHeight (MINIMUM_HEIGHT);
+  }
+}
+
 void DlgSettingsSegments::slotFillCorners (int state)
 {
   LOG4CPP_INFO_S ((*mainCat)) << "DlgSettingsSegments::slotFillCorner";
@@ -300,20 +371,39 @@ void DlgSettingsSegments::slotFillCorners (int state)
   updatePreview();
 }
 
-void DlgSettingsSegments::slotLineColor (const QString &)
+void DlgSettingsSegments::slotInactiveOpacity (const QString &)
 {
-  LOG4CPP_INFO_S ((*mainCat)) << "DlgSettingsSegments::slotLineColor";
+  LOG4CPP_INFO_S ((*mainCat)) << "DlgSettingsSegments::slotInactiveOpacity";
 
-  m_modelSegmentsAfter->setLineColor((ColorPalette) m_cmbLineColor->currentData().toInt());
+  InactiveOpacity inactiveOpacity = static_cast<InactiveOpacity> (m_cmbInactiveOpacity->currentData().toInt());
+  m_modelSegmentsAfter->setInactiveOpacity(inactiveOpacity);
   updateControls();
   updatePreview();
 }
 
-void DlgSettingsSegments::slotLineWidth (int lineWidth)
+void DlgSettingsSegments::slotLineColor (const QString &)
 {
-  LOG4CPP_INFO_S ((*mainCat)) << "DlgSettingsSegments::slotLineWidth";
+  LOG4CPP_INFO_S ((*mainCat)) << "DlgSettingsSegments::slotLineColor";
 
-  m_modelSegmentsAfter->setLineWidth(lineWidth);
+  m_modelSegmentsAfter->setLineColor(static_cast<ColorPalette> (m_cmbLineColor->currentData().toInt()));
+  updateControls();
+  updatePreview();
+}
+
+void DlgSettingsSegments::slotLineWidthActive (int lineWidthActive)
+{
+  LOG4CPP_INFO_S ((*mainCat)) << "DlgSettingsSegments::slotLineWidthActive";
+
+  m_modelSegmentsAfter->setLineWidthActive(lineWidthActive);
+  updateControls();
+  updatePreview();
+}
+
+void DlgSettingsSegments::slotLineWidthInactive (int lineWidthInactive)
+{
+  LOG4CPP_INFO_S ((*mainCat)) << "DlgSettingsSegments::slotLineWidthInactive";
+
+  m_modelSegmentsAfter->setLineWidthInactive(lineWidthInactive);
   updateControls();
   updatePreview();
 }
@@ -336,6 +426,11 @@ void DlgSettingsSegments::slotPointSeparation (const QString &pointSeparation)
   updatePreview();
 }
 
+void DlgSettingsSegments::slotWhatsThis ()
+{
+  QWhatsThis::enterWhatsThisMode();
+}
+
 void DlgSettingsSegments::updateControls()
 {
   enableOk (true);
@@ -346,43 +441,63 @@ void DlgSettingsSegments::updatePreview()
   LOG4CPP_INFO_S ((*mainCat)) << "DlgSettingsSegments::updatePreview"
                               << " loading=" << (m_loading ? "true" : "false");
 
+  updatePreviewOneScene (m_scenePreviewActive,
+                         m_segmentsActive,
+                         m_pointsActive,
+                         HOVER_ACTIVE);
+  updatePreviewOneScene (m_scenePreviewInactive,
+                         m_segmentsInactive,
+                         m_pointsInactive,
+                         HOVER_INACTIVE);
+}
+
+void DlgSettingsSegments::updatePreviewOneScene (QGraphicsScene *scenePreview,
+                                                 QList<Segment*> &segments,
+                                                 GraphicsPoints &points,
+                                                 HoverState hoverState)
+{
   const QString ARBITRARY_IDENTIFIER ("");
   const QColor COLOR (Qt::blue);
   const int RADIUS = 5;
-  GeometryWindow *NULL_GEOMETRY_WINDOW = 0;
+  GeometryWindow *NULL_GEOMETRY_WINDOW = nullptr;
+  const bool NO_DIALOG = false; // If true then dueling modal dialogs will trigger infinite loops in QSpinBox up/down
 
   if (!m_loading) {
 
-    SegmentFactory segmentFactory (*m_scenePreview,
+    SegmentFactory segmentFactory (*scenePreview,
                                    mainWindow().isGnuplot());
 
-    clearPoints();
-    segmentFactory.clearSegments (m_segments);
+    clearPointsOneScene (points);
+    segmentFactory.clearSegments (segments);
 
     // Create new segments
     segmentFactory.makeSegments (createPreviewImage(),
                                  *m_modelSegmentsAfter,
-                                 m_segments);
+                                 segments,
+                                 NO_DIALOG);
 
     // Make the segment visible
     QList<Segment*>::iterator itrS;
-    for (itrS = m_segments.begin(); itrS != m_segments.end(); itrS++) {
+    for (itrS = segments.begin(); itrS != segments.end(); itrS++) {
       Segment *segment = *itrS;
-      segment->slotHover (true);
+
+      segment->slotHover (hoverState == HOVER_ACTIVE);
+      segment->lockHoverState ();
     }
 
     // Create some points
     PointStyle pointStyle (POINT_SHAPE_CROSS,
                            RADIUS,
-                           BRUSH_WIDTH,
+                           qFloor (BRUSH_WIDTH),
                            COLOR_PALETTE_BLUE);
     QPolygonF polygon = pointStyle.polygon();
-    QList<QPoint> points = segmentFactory.fillPoints (*m_modelSegmentsAfter,
-                                                      m_segments);
+    QList<QPoint> pointsFill = segmentFactory.fillPoints (*m_modelSegmentsAfter,
+                                                          segments);
+
     QList<QPoint>::iterator itrP;
-    for (itrP = points.begin(); itrP != points.end(); itrP++) {
+    for (itrP = pointsFill.begin(); itrP != pointsFill.end(); itrP++) {
       QPoint pos = *itrP;
-      GraphicsPoint *graphicsPoint = new GraphicsPoint (*m_scenePreview,
+      GraphicsPoint *graphicsPoint = new GraphicsPoint (*scenePreview,
                                                         ARBITRARY_IDENTIFIER,
                                                         pos,
                                                         COLOR,
@@ -390,7 +505,7 @@ void DlgSettingsSegments::updatePreview()
                                                         BRUSH_WIDTH,
                                                         NULL_GEOMETRY_WINDOW);
 
-      m_points.push_back (graphicsPoint);
+      points.push_back (graphicsPoint);
     }
   }
 }
