@@ -37,35 +37,156 @@ void ExportFileRelations::exportAllPerLineXThetaValuesMerged (const DocumentMode
                                                               const QStringList &curvesIncluded,
                                                               const QString &delimiter,
                                                               const Transformation &transformation,
-                                                              QTextStream &str) const
+                                                              bool isLogXTheta,
+                                                              bool isLogYRadius,
+                                                              QTextStream &str,
+                                                              unsigned int &numWritesSoFar) const
 {
   LOG4CPP_INFO_S ((*mainCat)) << "ExportFileRelations::exportAllPerLineXThetaValuesMerged";
 
-  int curveCount = curvesIncluded.count();
   int maxColumnSize = maxColumnSizeAllocation (modelExportOverride,
                                                document,
                                                transformation,
+                                               isLogXTheta,
+                                               isLogYRadius,
                                                curvesIncluded);
 
   // Skip if every curve was a function
   if (maxColumnSize > 0) {
 
-    QVector<QVector<QString*> > xThetaYRadiusValues (COLUMNS_PER_CURVE * curveCount, QVector<QString*> (maxColumnSize));
-    initializeXThetaYRadiusValues (curvesIncluded,
-                                   xThetaYRadiusValues);
+    if (modelExportOverride.pointsSelectionRelations() == EXPORT_POINTS_SELECTION_RELATIONS_INTERPOLATE) {
+
+      exportAllPerLineXThetaValuesMergedMultiplePass (maxColumnSize,
+                                                      modelExportOverride,
+                                                      document,
+                                                      modelMainWindow,
+                                                      curvesIncluded,
+                                                      delimiter,
+                                                      transformation,
+                                                      isLogXTheta,
+                                                      isLogYRadius,
+                                                      str,
+                                                      numWritesSoFar);
+
+    } else {
+
+      exportAllPerLineXThetaValuesMergedOnePass (maxColumnSize,
+                                                 modelExportOverride,
+                                                 document,
+                                                 modelMainWindow,
+                                                 curvesIncluded,
+                                                 delimiter,
+                                                 transformation,
+                                                 isLogXTheta,
+                                                 isLogYRadius,
+                                                 str,
+                                                 numWritesSoFar);
+    }
+  }
+}
+
+void ExportFileRelations::exportAllPerLineXThetaValuesMergedMultiplePass (int maxColumnSize,
+                                                                          const DocumentModelExportFormat &modelExportOverride,
+                                                                          const Document &document,
+                                                                          const MainWindowModel &modelMainWindow,
+                                                                          const QStringList &curvesIncluded,
+                                                                          const QString &delimiter,
+                                                                          const Transformation &transformation,
+                                                                          bool isLogXTheta,
+                                                                          bool isLogYRadius,
+                                                                          QTextStream &str,
+                                                                          unsigned int &numWritesSoFar) const
+{
+  // For interpolation of relations in general a single set of x/theta values cannot be created that work for every
+  // relation curve, since one curve may have M y/radius values for a specific x/radius while another curve has
+  // N y/radius values for that same x/radius value. So we export each curve into memory separately and then merge
+  // the results. Why? Since the methods called from this method all assume a single set of x/theta values can be
+  // used for all curves
+
+  const int CURVE_COUNT_PER_PASS = 1;
+
+  int curveCount = curvesIncluded.count ();
+
+  QVector<QVector<QString*> > xThetaYRadiusValuesAll (COLUMNS_PER_CURVE * curveCount, QVector<QString*> (maxColumnSize));
+
+  initializeXThetaYRadiusValues (curvesIncluded,
+                                 xThetaYRadiusValuesAll);
+
+  // One pass per curve
+  int colX = 0, colY = colX + 1;
+  for (int c = 0; c < curvesIncluded.count (); c++) {
+    QString curve = curvesIncluded [c];
+
+    QStringList curvesIncludedInPass;
+    curvesIncludedInPass << curve;
+    ENGAUGE_ASSERT (curvesIncludedInPass.count () == CURVE_COUNT_PER_PASS);
+
+    QVector<QVector<QString*> > xThetaYRadiusValuesOne (COLUMNS_PER_CURVE * CURVE_COUNT_PER_PASS, QVector<QString*> (maxColumnSize));
+
+    initializeXThetaYRadiusValues (curvesIncludedInPass,
+                                   xThetaYRadiusValuesOne);
     loadXThetaYRadiusValues (modelExportOverride,
                              document,
                              modelMainWindow,
-                             curvesIncluded,
+                             curvesIncludedInPass,
                              transformation,
-                             xThetaYRadiusValues);
-    outputXThetaYRadiusValues (modelExportOverride,
-                               curvesIncluded,
-                               xThetaYRadiusValues,
-                               delimiter,
-                               str);
-    destroy2DArray (xThetaYRadiusValues);
+                             isLogXTheta,
+                             isLogYRadius,
+                             xThetaYRadiusValuesOne);
+
+    // Merge one curve array into all curves array
+    for (int row = 0; row < maxColumnSize; row++) {
+      *(xThetaYRadiusValuesAll [colX] [row]) = *(xThetaYRadiusValuesOne [0] [row]);
+      *(xThetaYRadiusValuesAll [colY] [row]) = *(xThetaYRadiusValuesOne [1] [row]);
+    }
+
+    destroy2DArray (xThetaYRadiusValuesOne);
+
+    colX += 2;
+    colY += 2;
   }
+
+  outputXThetaYRadiusValues (modelExportOverride,
+                             curvesIncluded,
+                             xThetaYRadiusValuesAll,
+                             delimiter,
+                             str,
+                             numWritesSoFar);
+  destroy2DArray (xThetaYRadiusValuesAll);
+}
+
+void ExportFileRelations::exportAllPerLineXThetaValuesMergedOnePass (int maxColumnSize,
+                                                                     const DocumentModelExportFormat &modelExportOverride,
+                                                                     const Document &document,
+                                                                     const MainWindowModel &modelMainWindow,
+                                                                     const QStringList &curvesIncluded,
+                                                                     const QString &delimiter,
+                                                                     const Transformation &transformation,
+                                                                     bool isLogXTheta,
+                                                                     bool isLogYRadius,
+                                                                     QTextStream &str,
+                                                                     unsigned int &numWritesSoFar) const
+{
+  int curveCount = curvesIncluded.count ();
+
+  QVector<QVector<QString*> > xThetaYRadiusValues (COLUMNS_PER_CURVE * curveCount, QVector<QString*> (maxColumnSize));
+  initializeXThetaYRadiusValues (curvesIncluded,
+                                 xThetaYRadiusValues);
+  loadXThetaYRadiusValues (modelExportOverride,
+                           document,
+                           modelMainWindow,
+                           curvesIncluded,
+                           transformation,
+                           isLogXTheta,
+                           isLogYRadius,
+                           xThetaYRadiusValues);
+  outputXThetaYRadiusValues (modelExportOverride,
+                             curvesIncluded,
+                             xThetaYRadiusValues,
+                             delimiter,
+                             str,
+                             numWritesSoFar);
+  destroy2DArray (xThetaYRadiusValues);
 }
 
 void ExportFileRelations::exportOnePerLineXThetaValuesMerged (const DocumentModelExportFormat &modelExportOverride,
@@ -74,7 +195,10 @@ void ExportFileRelations::exportOnePerLineXThetaValuesMerged (const DocumentMode
                                                               const QStringList &curvesIncluded,
                                                               const QString &delimiter,
                                                               const Transformation &transformation,
-                                                              QTextStream &str) const
+                                                              bool isLogXTheta,
+                                                              bool isLogYRadius,
+                                                              QTextStream &str,
+                                                              unsigned int &numWritesSoFar) const
 {
   LOG4CPP_INFO_S ((*mainCat)) << "ExportFileRelations::exportOnePerLineXThetaValuesMerged";
 
@@ -89,7 +213,10 @@ void ExportFileRelations::exportOnePerLineXThetaValuesMerged (const DocumentMode
                                         QStringList (curveIncluded),
                                         delimiter,
                                         transformation,
-                                        str);
+                                        isLogXTheta,
+                                        isLogYRadius,
+                                        str,
+                                        numWritesSoFar);
   }
 }
 
@@ -97,10 +224,20 @@ void ExportFileRelations::exportToFile (const DocumentModelExportFormat &modelEx
                                         const Document &document,
                                         const MainWindowModel &modelMainWindow,
                                         const Transformation &transformation,
-                                        QTextStream &str) const
+                                        QTextStream &str,
+                                        unsigned int &numWritesSoFar,
+                                        bool &isOverrun) const
 {
   LOG4CPP_INFO_S ((*mainCat)) << "ExportFileRelations::exportToFile";
 
+  // There are no known ways for relations to overflow the maximum points limit so
+  // this initialization value will not be overwritten
+  isOverrun = false;
+
+  // Log coordinates must be temporarily transformed to linear coordinates
+  bool isLogXTheta = (document.modelCoords().coordScaleXTheta() == COORD_SCALE_LOG);
+  bool isLogYRadius = (document.modelCoords().coordScaleYRadius() == COORD_SCALE_LOG);
+
   // Identify curves to be included
   QStringList curvesIncluded = curvesToInclude (modelExportOverride,
                                                 document,
@@ -120,7 +257,10 @@ void ExportFileRelations::exportToFile (const DocumentModelExportFormat &modelEx
                                         curvesIncluded,
                                         delimiter,
                                         transformation,
-                                        str);
+                                        isLogXTheta,
+                                        isLogYRadius,
+                                        str,
+                                        numWritesSoFar);
   } else {
     exportOnePerLineXThetaValuesMerged (modelExportOverride,
                                         document,
@@ -128,7 +268,10 @@ void ExportFileRelations::exportToFile (const DocumentModelExportFormat &modelEx
                                         curvesIncluded,
                                         delimiter,
                                         transformation,
-                                        str);
+                                        isLogXTheta,
+                                        isLogYRadius,
+                                        str,
+                                        numWritesSoFar);
   }
 }
 
@@ -206,6 +349,8 @@ void ExportFileRelations::loadXThetaYRadiusValues (const DocumentModelExportForm
                                                    const MainWindowModel &modelMainWindow,
                                                    const QStringList &curvesIncluded,
                                                    const Transformation &transformation,
+                                                   bool isLogXTheta,
+                                                   bool isLogYRadius,
                                                    QVector<QVector<QString*> > &xThetaYRadiusValues) const
 {
   LOG4CPP_INFO_S ((*mainCat)) << "ExportFileRelations::loadXThetaYRadiusValues";
@@ -240,6 +385,8 @@ void ExportFileRelations::loadXThetaYRadiusValues (const DocumentModelExportForm
                                                           modelExportOverride.pointsIntervalUnitsRelations(),
                                                           lineStyle.curveConnectAs(),
                                                           transformation,
+                                                          isLogXTheta,
+                                                          isLogYRadius,
                                                           points);
 
       if (curve->curveStyle().lineStyle().curveConnectAs() == CONNECT_AS_RELATION_SMOOTH) {
@@ -251,7 +398,9 @@ void ExportFileRelations::loadXThetaYRadiusValues (const DocumentModelExportForm
                                                            ordinals,
                                                            xThetaYRadiusValues [colXTheta],
                                                            xThetaYRadiusValues [colYRadius],
-                                                           transformation);
+                                                           transformation,
+                                                           isLogXTheta,
+                                                           isLogYRadius);
 
       } else {
 
@@ -275,7 +424,9 @@ void ExportFileRelations::loadXThetaYRadiusValuesForCurveInterpolatedSmooth (con
                                                                              const ExportValuesOrdinal &ordinals,
                                                                              QVector<QString*> &xThetaValues,
                                                                              QVector<QString*> &yRadiusValues,
-                                                                             const Transformation &transformation) const
+                                                                             const Transformation &transformation,
+                                                                             bool isLogXTheta,
+                                                                             bool isLogYRadius) const
 {
   LOG4CPP_INFO_S ((*mainCat)) << "ExportFileRelations::loadXThetaYRadiusValuesForCurveInterpolatedSmooth";
 
@@ -285,6 +436,8 @@ void ExportFileRelations::loadXThetaYRadiusValuesForCurveInterpolatedSmooth (con
 
   ordinalsSmooth.loadSplinePairsWithTransformation (points,
                                                     transformation,
+                                                    isLogXTheta,
+                                                    isLogYRadius,
                                                     t,
                                                     xy);
 
@@ -387,6 +540,8 @@ void ExportFileRelations::loadXThetaYRadiusValuesForCurveRaw (const DocumentMode
 int ExportFileRelations::maxColumnSizeAllocation (const DocumentModelExportFormat &modelExport,
                                                   const Document &document,
                                                   const Transformation &transformation,
+                                                  bool isLogXTheta,
+                                                  bool isLogYRadius,
                                                   const QStringList &curvesIncluded) const
 {
   LOG4CPP_INFO_S ((*mainCat)) << "ExportFileRelations::maxColumnSizeAllocation";
@@ -411,11 +566,14 @@ int ExportFileRelations::maxColumnSizeAllocation (const DocumentModelExportForma
 
       const LineStyle &lineStyle = document.modelCurveStyles().lineStyle(curveName);
 
+
       // Interpolation. Points are taken approximately every every modelExport.pointsIntervalRelations
       ExportValuesOrdinal ordinals = ordinalsAtIntervals (modelExport.pointsIntervalRelations(),
                                                           modelExport.pointsIntervalUnitsRelations(),
                                                           lineStyle.curveConnectAs(),
                                                           transformation,
+                                                          isLogXTheta,
+                                                          isLogYRadius,
                                                           points);
 
       maxColumnSize = qMax (maxColumnSize,
@@ -430,6 +588,8 @@ ExportValuesOrdinal ExportFileRelations::ordinalsAtIntervals (double pointsInter
                                                               ExportPointsIntervalUnits pointsIntervalUnits,
                                                               CurveConnectAs curveConnectAs,
                                                               const Transformation &transformation,
+                                                              bool isLogXTheta,
+                                                              bool isLogYRadius,
                                                               const Points &points) const
 {
   LOG4CPP_INFO_S ((*mainCat)) << "ExportFileRelations::ordinalsAtIntervals";
@@ -439,6 +599,8 @@ ExportValuesOrdinal ExportFileRelations::ordinalsAtIntervals (double pointsInter
 
       return ordinalsAtIntervalsSmoothGraph (pointsIntervalRelations,
                                              transformation,
+                                             isLogXTheta,
+                                             isLogYRadius,
                                              points);
 
     } else {
@@ -466,6 +628,8 @@ ExportValuesOrdinal ExportFileRelations::ordinalsAtIntervals (double pointsInter
 
 ExportValuesOrdinal ExportFileRelations::ordinalsAtIntervalsSmoothGraph (double pointsIntervalRelations,
                                                                          const Transformation &transformation,
+                                                                         bool isLogXTheta,
+                                                                         bool isLogYRadius,
                                                                          const Points &points) const
 {
   LOG4CPP_INFO_S ((*mainCat)) << "ExportFileRelations::ordinalsAtIntervalsSmoothGraph";
@@ -482,6 +646,8 @@ ExportValuesOrdinal ExportFileRelations::ordinalsAtIntervalsSmoothGraph (double
 
     ordinalsSmooth.loadSplinePairsWithTransformation (points,
                                                       transformation,
+                                                      isLogXTheta,
+                                                      isLogYRadius,
                                                       t,
                                                       xy);
 
@@ -570,14 +736,17 @@ void ExportFileRelations::outputXThetaYRadiusValues (const DocumentModelExportFo
                                                      const QStringList &curvesIncluded,
                                                      QVector<QVector<QString*> > &xThetaYRadiusValues,
                                                      const QString &delimiter,
-                                                     QTextStream &str) const
+                                                     QTextStream &str,
+                                                     unsigned int &numWritesSoFar) const
 {
   LOG4CPP_INFO_S ((*mainCat)) << "ExportFileRelations::outputXThetaYRadiusValues";
 
   // Header
   if (modelExportOverride.header() != EXPORT_HEADER_NONE) {
+    insertLineSeparator (numWritesSoFar == 0,
+                         modelExportOverride.header (),
+                         str);
     if (modelExportOverride.header() == EXPORT_HEADER_GNUPLOT) {
-      str << curveSeparator(str.string());
       str << gnuplotComment();
     }
     QString delimiterForRow;
@@ -591,15 +760,20 @@ void ExportFileRelations::outputXThetaYRadiusValues (const DocumentModelExportFo
     str << "\n";
   }
 
+  // Table body
   for (int row = 0; row < xThetaYRadiusValues [0].count(); row++) {
 
     QString delimiterForRow;
     for (int col = 0; col < xThetaYRadiusValues.count(); col++) {
 
-      str << delimiterForRow << *(xThetaYRadiusValues [col] [row]);
+      QString xThetaString = *(xThetaYRadiusValues [col] [row]);
+      str << delimiterForRow << wrapInDoubleQuotesIfNeeded (modelExportOverride,
+                                                            xThetaString);
       delimiterForRow = delimiter;
     }
 
     str << "\n";
   }
+
+  ++numWritesSoFar;
 }
