@@ -7,6 +7,9 @@
 #include "EngaugeAssert.h"
 #include "FormatDateTime.h"
 #include "Logger.h"
+#include <QDateTime>
+#include <qmath.h>
+#include <QTimeZone>
 
 FormatDateTime::FormatDateTime()
 {
@@ -28,7 +31,7 @@ bool FormatDateTime::ambiguityBetweenDateAndTime (CoordUnitsDate coordUnitsDate,
       coordUnitsTime != COORD_UNITS_TIME_SKIP) {
 
     // See if there is just a single number
-    QStringList fields = string.trimmed().split(QRegExp ("[/- :]"));
+    QStringList fields = string.trimmed().split(QRegularExpression ("[/- :]"));
 
     if (fields.count() == 1) {
 
@@ -89,7 +92,10 @@ void FormatDateTime::dateTimeLookup (const FormatsDate &formatsDateAll,
                                                             string)) {
 
             success = true;
-            value = dt.toTime_t();
+            // Convert using local time to prevent addition of utc offset. Number of seconds since 1970 epoch
+            // is used with 64 bits resolution, versus time_t with only 32 bits resolution (and limites to 1970 to 2038).
+            // Time value is negative for pre-epoch. Grep for fromSecsSinceEpoch in this same file
+            value = toSecsSinceEpoch (dt.toLocalTime ());
             iterating = false; // Stop iterating
 
             LOG4CPP_INFO_S ((*mainCat)) << "FormatDateTime::dateTimeLookup"
@@ -101,8 +107,8 @@ void FormatDateTime::dateTimeLookup (const FormatsDate &formatsDateAll,
           }
         } else {
 
-          QRegExp reg (formatDateTime);
-          if (reg.exactMatch(string)) {
+          QRegularExpression reg (formatDateTime);
+          if (reg.match(string).hasMatch()) {
 
             success = true; // Note that value does not get set in QRegExp case
             iterating = false; // Stop iterating
@@ -131,9 +137,17 @@ QString FormatDateTime::formatOutput (CoordUnitsDate coordUnitsDate,
   QString format = m_formatsDateFormat [coordUnitsDate] + " " + m_formatsTimeFormat [coordUnitsTime];
   format = format.trimmed();
 
-  QDateTime dt = QDateTime::fromTime_t (value);
+  // We are using 64 bits resolution on seconds from epoch rather than time_t which has 32 bits resolution (and
+  // is therefore limited to 1970 to 2038). Time value is negative for pre-epoch. Grep for toSecsSinceEpoch in this same file
+  QDateTime dt;
+  dt = fromSecsSinceEpoch (static_cast<qint64> (value));
 
-  return dt.toString (format);
+  return dt.toLocalTime ().toString (format); // Convert using local time to prevent addition of utc offset
+}
+
+QDateTime FormatDateTime::fromSecsSinceEpoch (qint64 secs) const
+{
+  return QDateTime::fromMSecsSinceEpoch (secs * 1000);
 }
 
 void FormatDateTime::loadFormatsFormat()
@@ -312,7 +326,7 @@ void FormatDateTime::loadFormatsParseIncomplete()
 {
   LOG4CPP_INFO_S ((*mainCat)) << "FormatDateTime::loadFormatsParseIncomplete";
 
-  QStringList skip, day, dayMonth, month, monthDay, monthDayYear, year, yearMonth, yearMonthDay;
+  QStringList skip, day, dayMonth, dayMonthYear, month, monthDay, monthDayYear, year, yearMonth, yearMonthDay;
 
   // COORD_UNITS_DATE_SKIP and COORD_UNITS_TIME_SKIP allow date/time respectively even when skipped,
   // although there can be ambiguity with between COORD_UNITS_DATE_MONTH_DAY_YEAR and COORD_UNITS_DATE_DAY_MONTH_YEAR
@@ -330,6 +344,10 @@ void FormatDateTime::loadFormatsParseIncomplete()
            << "\\d{1,2}/[a-zA-Z]{1,12}/"
            << "\\d{1,2}-[a-zA-Z]{1,12}-"
            << "\\d{1,2} [a-zA-Z]{1,12} ";
+  dayMonthYear << "\\d{1,2}/\\d{1,2}/\\d{1,4}"
+               << "\\d{1,2}/\\d{1,2}/\\d{1,4} "
+               << "\\d{1,2}-\\d{1,2}-\\d{1,4}"
+               << "\\d{1,2}-\\d{1,2}-\\d{1,4} ";
   month << "\\d{1,2}"
         << "\\d{1,2}/"
         << "[a-zA-Z]{1,12}"
@@ -371,16 +389,20 @@ void FormatDateTime::loadFormatsParseIncomplete()
             << "\\d{4} [a-zA-Z]{1,12}"
             << "\\d{4} [a-zA-Z]{1,12} ";
   yearMonthDay << "\\d{4}/\\d{1,2}/\\d{1,2}"
+               << "\\d{4}/\\d{1,2}/\\d{1,2} "    
                << "\\d{4}/\\d{1,2}-\\d{1,2}"
+               << "\\d{4}/\\d{1,2}-\\d{1,2} "    
                << "\\d{4} \\d{1,2} \\d{1,2}"
                << "\\d{4}/[a-zA-Z]{1,12}/\\d{1,2}"
-               << "\\d{4}-[a-zA-Z]{1,12}-\\d{1,2}";
+               << "\\d{4}/[a-zA-Z]{1,12}/\\d{1,2} "    
+               << "\\d{4}-[a-zA-Z]{1,12}-\\d{1,2}"
+               << "\\d{4}-[a-zA-Z]{1,12}-\\d{1,2} ";    
 
   // For every entry, the possible states leading up to the Acceptable states in m_formatsDateParseIncomplete are all included.
   // Potential day-month ambiguity for COORD_UNITS_DATE_SKIP gets treated as month/day.
   m_formatsDateParseIncomplete [COORD_UNITS_DATE_SKIP] = skip + month + monthDay + monthDayYear + year + yearMonth + yearMonthDay;
   m_formatsDateParseIncomplete [COORD_UNITS_DATE_MONTH_DAY_YEAR] = skip + month + monthDay + monthDayYear + year + yearMonth + yearMonthDay;
-  m_formatsDateParseIncomplete [COORD_UNITS_DATE_DAY_MONTH_YEAR] = skip + day + dayMonth + year + yearMonth + yearMonthDay;
+  m_formatsDateParseIncomplete [COORD_UNITS_DATE_DAY_MONTH_YEAR] = skip + day + dayMonth + dayMonthYear + year + yearMonth + yearMonthDay;
   m_formatsDateParseIncomplete [COORD_UNITS_DATE_YEAR_MONTH_DAY] = skip + year + yearMonth + yearMonthDay;
 
   ENGAUGE_ASSERT (m_formatsDateParseIncomplete.count () == NUM_COORD_UNITS_DATE);
@@ -455,7 +477,7 @@ QValidator::State FormatDateTime::parseInput (CoordUnitsDate coordUnitsDate,
       
       // Not acceptable, but perhaps it is just incomplete
       dateTimeLookup (m_formatsDateParseIncomplete,
-                    m_formatsTimeParseIncomplete,
+                      m_formatsTimeParseIncomplete,
                       coordUnitsDate,
                       coordUnitsTime,
                       string,
@@ -472,3 +494,8 @@ QValidator::State FormatDateTime::parseInput (CoordUnitsDate coordUnitsDate,
 
   return state;
 }
+
+qint64 FormatDateTime::toSecsSinceEpoch (const QDateTime &dt) const
+{
+  return dt.toMSecsSinceEpoch () / static_cast<qint64> (1000);
+}
