@@ -4,6 +4,7 @@
  * LICENSE or go to gnu.org/licenses for details. Distribution requires prior written permission.     *
  ******************************************************************************************************/
 
+#include "Document.h"
 #include "DocumentModelCoords.h"
 #include "DocumentModelGridDisplay.h"
 #include "EngaugeAssert.h"
@@ -11,6 +12,7 @@
 #include "GraphicsArcItem.h"
 #include "GridLineFactory.h"
 #include "GridLineLimiter.h"
+#include "GridLineNormalize.h"
 #include "GridLines.h"
 #include "GridLineStyle.h"
 #include "Logger.h"
@@ -26,10 +28,7 @@ const int Z_VALUE_IN_FRONT = 100;
 // To emphasize that the axis lines are still there, we make these checker somewhat transparent
 const double CHECKER_OPACITY = 0.6;
 
-const double PI = 3.1415926535;
-const double TWO_PI = 2.0 * PI;
-const double DEGREES_TO_RADIANS = PI / 180.0;
-const double RADIANS_TO_TICS = 5760 / TWO_PI;
+const double RADIANS_TO_TICS = 360 * 16 / (2.0 * M_PI);
 
 GridLineFactory::GridLineFactory(QGraphicsScene &scene,
                                  const DocumentModelCoords &modelCoords) :
@@ -100,7 +99,7 @@ GridLine *GridLineFactory::createGridLine (double xFrom,
   // Loop through steps. Final step i=NUM_STEPS does final processing if a segment is active
   for (int i = 0; i <= NUM_STEPS; i++) {
 
-    double s = (double) i / (double) NUM_STEPS;
+    double s = double (i) / double (NUM_STEPS);
 
     // Interpolate coordinates assuming normal linear scaling
     double xGraph = (1.0 - s) * xFrom + s * xTo;
@@ -152,6 +151,7 @@ GridLine *GridLineFactory::createGridLine (double xFrom,
 }
 
 void GridLineFactory::createGridLinesForEvenlySpacedGrid (const DocumentModelGridDisplay &modelGridDisplay,
+                                                          const Document &document,
                                                           const MainWindowModel &modelMainWindow,
                                                           const Transformation &transformation,
                                                           GridLines &gridLines)
@@ -168,26 +168,57 @@ void GridLineFactory::createGridLinesForEvenlySpacedGrid (const DocumentModelGri
     double stepY  = modelGridDisplay.stepY  ();
     double stopX  = modelGridDisplay.stopX  ();
     double stopY  = modelGridDisplay.stopY  ();
+    unsigned int numX = modelGridDisplay.countX ();
+    unsigned int numY = modelGridDisplay.countY ();
+
+    // Linear or log?
+    bool isLinearX = (m_modelCoords.coordScaleXTheta() == COORD_SCALE_LINEAR);
+    bool isLinearY = (m_modelCoords.coordScaleYRadius() == COORD_SCALE_LINEAR);
+
+    GridLineNormalize normalize (modelMainWindow);
+    normalize.normalize (isLinearX,
+                         modelGridDisplay.disableX(),
+                         startX,
+                         stepX,
+                         stopX,
+                         numX);
+    normalize.normalize (isLinearY,
+                         modelGridDisplay.disableY(),
+                         startY,
+                         stepY,
+                         stopY,
+                         numY);
 
     // Limit the number of grid lines. This is a noop if the limit is not exceeded
     GridLineLimiter gridLineLimiter;
-    stepX = gridLineLimiter.limitedStepXTheta (m_modelCoords,
-                                               modelMainWindow,
-                                               modelGridDisplay);
-    stepY = gridLineLimiter.limitedStepYRange (m_modelCoords,
-                                               modelMainWindow,
-                                               modelGridDisplay);
+    gridLineLimiter.limitForXTheta (document,
+                                    transformation,
+                                    m_modelCoords,
+                                    modelMainWindow,
+                                    startX,
+                                    stepX,
+                                    stopX,
+                                    numX);
+    gridLineLimiter.limitForYRadius (document,
+                                     transformation,
+                                     m_modelCoords,
+                                     modelMainWindow,
+                                     startY,
+                                     stepY,
+                                     stopY,
+                                     numY);
 
     // Apply if possible
-    if (stepX != 0 &&
-        stepY != 0) {
+    if (stepX > (isLinearX ? 0 : 1) &&
+        stepY > (isLinearY ? 0 : 1) &&
+        (isLinearX || (startX > 0)) &&
+        (isLinearY || (startY > 0))) {
 
       QColor color (ColorPaletteToQColor (modelGridDisplay.paletteColor()));
       QPen pen (QPen (color,
-                      GRID_LINE_WIDTH,
+                      modelGridDisplay.lineWidth (),
                       GRID_LINE_STYLE));
 
-      bool isLinearX = (m_modelCoords.coordScaleXTheta() == COORD_SCALE_LINEAR);
       for (double x = startX; x <= stopX; (isLinearX ? x += stepX : x *= stepX)) {
 
         GridLine *gridLine = createGridLine (x, startY, x, stopY, transformation);
@@ -195,7 +226,6 @@ void GridLineFactory::createGridLinesForEvenlySpacedGrid (const DocumentModelGri
         gridLines.add (gridLine);
       }
 
-      bool isLinearY = (m_modelCoords.coordScaleYRadius() == COORD_SCALE_LINEAR);
       for (double y = startY; y <= stopY; (isLinearY ? y += stepY : y *= stepY)) {
 
         GridLine *gridLine = createGridLine (startX, y, stopX, y, transformation);
@@ -250,8 +280,9 @@ void GridLineFactory::createTransformAlign (const Transformation &transformation
                                                                                 posXRadiusY0AlignedScreen,
                                                                                 posX0YRadiusAlignedScreen);
 
+  // Use \n rather than endl to prevent compiler warning "nonnull argument t compared to null"
   LOG4CPP_INFO_S ((*mainCat)) << "GridLineFactory::createTransformAlign"
-                              << " transformation=" << QTransformToString (transformation.transformMatrix()).toLatin1().data() << endl
+                              << " transformation=" << QTransformToString (transformation.transformMatrix()).toLatin1().data() << "\n"
                               << " radiusLinearCartesian=" << radiusLinearCartesian
                               << " posXRadiusY0Screen=" << QPointFToString (posXRadiusY0Screen).toLatin1().data()
                               << " posX0YRadiusScreen=" << QPointFToString (posX0YRadiusScreen).toLatin1().data()
@@ -277,10 +308,10 @@ QGraphicsItem *GridLineFactory::ellipseItem (const Transformation &transformatio
                                             posEndGraph);
 
   // Get the angles about the origin of the start and end points
-  double angleStart = posStartGraph.x() * DEGREES_TO_RADIANS;
-  double angleEnd = posEndGraph.x() * DEGREES_TO_RADIANS;
+  double angleStart = qDegreesToRadians (posStartGraph.x());
+  double angleEnd = qDegreesToRadians (posEndGraph.x());
   if (angleEnd < angleStart) {
-    angleEnd += TWO_PI;
+    angleEnd += 2.0 * M_PI;
   }
   double angleSpan = angleEnd - angleStart;
 
@@ -294,8 +325,8 @@ QGraphicsItem *GridLineFactory::ellipseItem (const Transformation &transformatio
                               << " posStartScreen=" << QPointFToString (posStartScreen).toLatin1().data()
                               << " posEndScreen=" << QPointFToString (posEndScreen).toLatin1().data()
                               << " posOriginScreen=" << QPointFToString (posOriginScreen).toLatin1().data()
-                              << " angleStart=" << angleStart / DEGREES_TO_RADIANS
-                              << " angleEnd=" << angleEnd / DEGREES_TO_RADIANS
+                              << " angleStart=" << qRadiansToDegrees (angleStart)
+                              << " angleEnd=" << qRadiansToDegrees (angleEnd)
                               << " transformation=" << transformation;
 
   // Compute rotate/shear transform that aligns linear cartesian graph coordinates with screen coordinates, and ellipse parameters.
@@ -316,8 +347,8 @@ QGraphicsItem *GridLineFactory::ellipseItem (const Transformation &transformatio
                        2 * ellipseXAxis,
                        2 * ellipseYAxis);
   GraphicsArcItem *item = new GraphicsArcItem (boundingRect);
-  item->setStartAngle (angleStart * RADIANS_TO_TICS);
-  item->setSpanAngle (angleSpan * RADIANS_TO_TICS);
+  item->setStartAngle (qFloor (angleStart * RADIANS_TO_TICS));
+  item->setSpanAngle (qFloor (angleSpan * RADIANS_TO_TICS));
 
   item->setTransform (transformAlign.transposed ().inverted ());
 
