@@ -11,6 +11,7 @@
 #include "Logger.h"
 #include <QDebug>
 #include <qmath.h>
+#include <QObject>
 #include <QtGlobal>
 #include "QtToString.h"
 #include "Transformation.h"
@@ -21,7 +22,6 @@ using namespace std;
 /// number of characters.
 const int PRECISION_DIGITS = 4;
 
-const double PI = 3.1415926535;
 const double ZERO_OFFSET_AFTER_LOG = 1; // Log of this value is zero
 
 Transformation::Transformation() :
@@ -93,11 +93,11 @@ QPointF Transformation::cartesianFromCartesianOrPolar (const DocumentModelCoords
       case COORD_UNITS_POLAR_THETA_DEGREES_MINUTES:
       case COORD_UNITS_POLAR_THETA_DEGREES_MINUTES_SECONDS:
       case COORD_UNITS_POLAR_THETA_DEGREES_MINUTES_SECONDS_NSEW:
-        angleRadians = posGraphIn.x () * PI / 180.0;
+        angleRadians = posGraphIn.x () * M_PI / 180.0;
         break;
 
       case COORD_UNITS_POLAR_THETA_GRADIANS:
-        angleRadians = posGraphIn.x () * PI / 200.0;
+        angleRadians = posGraphIn.x () * M_PI / 200.0;
         break;
 
       case COORD_UNITS_POLAR_THETA_RADIANS:
@@ -105,7 +105,7 @@ QPointF Transformation::cartesianFromCartesianOrPolar (const DocumentModelCoords
         break;
 
       case COORD_UNITS_POLAR_THETA_TURNS:
-        angleRadians = posGraphIn.x () * 2.0 * PI;
+        angleRadians = posGraphIn.x () * 2.0 * M_PI;
         break;
 
       default:
@@ -137,11 +137,11 @@ QPointF Transformation::cartesianOrPolarFromCartesian (const DocumentModelCoords
       case COORD_UNITS_POLAR_THETA_DEGREES_MINUTES:
       case COORD_UNITS_POLAR_THETA_DEGREES_MINUTES_SECONDS:
       case COORD_UNITS_POLAR_THETA_DEGREES_MINUTES_SECONDS_NSEW:
-        posGraphCartesianOrPolar.setX (angleRadians * 180.0 / PI);
+        posGraphCartesianOrPolar.setX (angleRadians * 180.0 / M_PI);
         break;
 
       case COORD_UNITS_POLAR_THETA_GRADIANS:
-        posGraphCartesianOrPolar.setX (angleRadians * 200.0 / PI);
+        posGraphCartesianOrPolar.setX (angleRadians * 200.0 / M_PI);
         break;
 
       case COORD_UNITS_POLAR_THETA_RADIANS:
@@ -149,7 +149,7 @@ QPointF Transformation::cartesianOrPolarFromCartesian (const DocumentModelCoords
         break;
 
       case COORD_UNITS_POLAR_THETA_TURNS:
-        posGraphCartesianOrPolar.setX (angleRadians / 2.0 / PI);
+        posGraphCartesianOrPolar.setX (angleRadians / (2.0 * M_PI));
         break;
 
       default:
@@ -166,12 +166,17 @@ QPointF Transformation::cartesianOrPolarFromCartesian (const DocumentModelCoords
 void Transformation::coordTextForStatusBar (QPointF cursorScreen,
                                             QString &coordsScreen,
                                             QString &coordsGraph,
-                                            QString &resolutionsGraph)
+                                            QString &resolutionsGraph,
+                                            bool usingScaleBar)
 {
   const int UNCONSTRAINED_FIELD_WIDTH = 0;
   const double X_DELTA_PIXELS = 1.0, Y_DELTA_PIXELS = 1.0;
   const char FORMAT = 'g';
 
+  QString needMoreText = (usingScaleBar ?
+                            QObject::tr ("Need scale bar") :
+                            QObject::tr ("Need more axis points"));
+
   if (cursorScreen.x() < 0 ||
       cursorScreen.y() < 0) {
 
@@ -225,7 +230,8 @@ void Transformation::coordTextForStatusBar (QPointF cursorScreen,
 
     } else {
 
-      coordsGraph = "<font color=\"red\">Need more axis points</font>";
+      coordsGraph = QString ("<font color=\"red\">%1</font>")
+        .arg (needMoreText);
       resolutionsGraph = coordsGraph;
 
     }
@@ -480,6 +486,7 @@ void Transformation::update (bool fileIsLoaded,
 
       updateTransformFromMatrices (ftor.matrixScreen(),
                                    ftor.matrixGraph());
+
     } else {
 
       m_transformIsDefined = false;
@@ -525,9 +532,7 @@ void Transformation::updateTransformFromMatrices (const QTransform &matrixScreen
                                 pointGraphLinearCart2.x(),
                                 pointGraphLinearCart0.y(),
                                 pointGraphLinearCart1.y(),
-                                pointGraphLinearCart2.y(),
-                                1.0,
-                                1.0);
+                                pointGraphLinearCart2.y());
 
   QPointF pointScreenRoundTrip0, pointScreenRoundTrip1, pointScreenRoundTrip2;
   transformRawGraphToScreen (pointGraphRaw0,
