@@ -11,11 +11,14 @@
 #include "CoordSystemIndex.h"
 #include "DigitizeStateAbstractBase.h"
 #include "DocumentAxesPointsRequired.h"
+#include "FittingCurveCoefficients.h"
 #include "GridLines.h"
+#include "Guidelines.h"
 #include "MainWindowModel.h"
 #include <QCursor>
 #include <QMainWindow>
 #include <QMap>
+#include <QPointF>
 #include <QUrl>
 #include "Transformation.h"
 #include "ZoomControl.h"
@@ -31,13 +34,14 @@ class DigitizeStateContext;
 class DlgSettingsAxesChecker;
 class DlgSettingsColorFilter;
 class DlgSettingsCoords;
-class DlgSettingsCurveAddRemove;
+class DlgSettingsCurveList;
 class DlgSettingsCurveProperties;
 class DlgSettingsDigitizeCurve;
 class DlgSettingsExportFormat;
 class DlgSettingsGeneral;
 class DlgSettingsGridDisplay;
 class DlgSettingsGridRemoval;
+class DlgSettingsGuideline;
 class DlgSettingsMainWindow;
 class DlgSettingsPointMatch;
 class DlgSettingsSegments;
@@ -53,11 +57,15 @@ class DocumentModelPointMatch;
 class DocumentModelSegments;
 class ExportToFile;
 class FileCmdScript;
+class FittingCurve;
+class FittingWindow;
 class GeometryWindow;
 class Ghosts;
 class GraphicsScene;
 class GraphicsView;
+#if !defined(OSX_DEBUG) && !defined(OSX_RELEASE)
 class HelpWindow;
+#endif
 class LoadImageFromUrl;
 class NetworkClient;
 class QAction;
@@ -66,40 +74,76 @@ class QCloseEvent;
 class QComboBox;
 class QDomDocument;
 class QGraphicsLineItem;
+class QGridLayout;
 class QMenu;
 class QPushButton;
 class QSettings;
+class QSignalMapper;
 class QTextStream;
 class QTimer;
 class QToolBar;
-class QVBoxLayout;
 class StatusBar;
 class TransformationStateContext;
 class TutorialDlg;
 class ViewPointStyle;
 class ViewSegmentFilter;
 
+extern const unsigned int MAX_RECENT_FILE_LIST_SIZE;
+
 /// Main window consisting of menu, graphics scene, status bar and optional toolbars as a Single Document Interface
 class MainWindow : public QMainWindow
 {
   Q_OBJECT
 
+  /// For simplifying this class by moving initialization to other classes
+  friend class CreateActions;
+  friend class CreateCentralWidget;
+  friend class CreateCommandStackShadow;
+  friend class CreateDockableWidgets;
+  friend class CreateFacade;
+  friend class CreateHelpWindow;
+  friend class CreateIcons;
+  friend class CreateLoadImage;
+  friend class CreateMenus;
+  friend class CreateNetwork;
+  friend class CreateScene;
+  friend class CreateSettingsDialogs;
+  friend class CreateStateContexts;
+  friend class CreateStatusBar;
+  friend class CreateToolBars;
+  friend class CreateTutorial;
+  friend class CreateZoomMaps;
+
+  /// For unit testing
+  friend class TestExport;
+  friend class TestGuidelines;
+  
 public:
   /// Single constructor.
   /// \param errorReportFile Optional error report file to be read at startup. Empty if unused. Incompatible with fileCmdScript
   /// \param fileCmdScriptFile Optional file command script file to be read at startup. Empty if unused. Incompatible with errorReportFile
+  /// \param isDropRegression True if files loaded at startup are for drag and drop testing
   /// \param isRegressionTest True if errorReportFile or fileCmdScript is for regression testing, in which case it is executed and the program exits
   /// \param isGnuplot True if diagnostic gnuplot files are generated for math-intense sections of the code. Used for development and debugging
   /// \param isReset True to reset all settings that would otherwise be restored from the previous execution of Engauge
+  /// \param isExportOnly True to export the loaded startup file and then exit
+  /// \param isExtractImageOnly True to extract the image from the loaded startup file and then exit
+  /// \param extractImageOnlyExtension File extension for extracted image for isExtractImageOnly
   /// \param loadStartupFiles Zero or more Engauge document files to load at startup. A separate instance of Engauge is created for each file
+  /// \param commandLineWithoutLoadStartupFiles Command line arguments without load startup files. Used for spawning additional processes
   /// \param parent Optional parent widget for this widget
   MainWindow(const QString &errorReportFile,
              const QString &fileCmdScriptFile,
+             bool isDropRegression,
              bool isRegressionTest,
              bool isGnuplot,
              bool isReset,
-             QStringList loadStartupFiles,
-             QWidget *parent = 0);
+             bool isExportOnly,
+             bool isExtractImageOnly,
+             const QString &extractImageOnlyExtension,
+             const QStringList &loadStartupFiles,
+             const QStringList &commandLineWithoutLoadStartupFiles,
+             QWidget *parent = nullptr);
   ~MainWindow();
 
   /// Close file. This is called from a file script command
@@ -117,8 +161,36 @@ public:
   /// Accessor for commands to process the Document.
   CmdMediator *cmdMediator();
 
+  /// Get DigitizeState. This should only be used to populate arguments in CmdAbstract constructor!
+  DigitizeState digitizeState () const;
+
   /// Catch secret keypresses
   virtual bool eventFilter(QObject *, QEvent *);
+  
+  /// Add a X/T Guideline
+  void guidelineAddXT (const QString &identifier,
+                       double xT);
+  
+  /// Add a Y/R Guideline
+  void guidelineAddYR (const QString &identifier,
+                       double yR);
+  
+  /// Move a X/T Guideline
+  void guidelineMoveXT (const QString &identifier,
+                        double xTAfter);
+  
+  /// Move a Y/R Guideline
+  void guidelineMoveYR (const QString &identifier,
+                        double yRAfter);
+  
+  /// Remove a X/T or Y/R Guideline
+  void guidelineRemove (const QString &identifier);
+  
+  /// True/false if guidelines are visible. Selectability is handled elsewhere
+  bool guidelinesAreVisible () const;
+  
+  /// Handle Guidelines active status toggle
+  void handleGuidelinesActiveChange (bool active);
 
   /// Background image that has been filtered for the current curve. This asserts if a curve-specific image is not being shown
   QImage imageFiltered () const;
@@ -129,6 +201,9 @@ public:
   /// Get method for main window model
   MainWindowModel modelMainWindow () const;
 
+  /// True if document scale is set using a scale bar, otherwise using axis points
+  bool modeMap () const;
+
   /// Intercept resize event so graphics scene can be appropriately resized when in Fill mode.
   void resizeEvent (QResizeEvent *event);
 
@@ -136,7 +211,7 @@ public:
   void saveErrorReportFileAndExit(const char *comment,
                                   const char *file,
                                   int line,
-                                  const char *context) const;
+                                  const char *context);
 
   /// Scene container for the QImage and QGraphicsItems.
   GraphicsScene &scene();
@@ -148,6 +223,9 @@ public:
   /// Curve name that is currently selected in m_cmbCurve.
   QString selectedGraphCurve () const;
 
+  /// Send signal to unit test framework indicating all commands have finished executing
+  void sendGong ();
+
   /// Processing performed after gui becomes available
   virtual void showEvent(QShowEvent *);
 
@@ -186,7 +264,7 @@ public:
   void updateSettingsCoords(const DocumentModelCoords &modelCoords);
 
   /// Update with new curves.
-  void updateSettingsCurveAddRemove (const CurvesGraphs &curvesGraphs);
+  void updateSettingsCurveList (const CurvesGraphs &curvesGraphs);
 
   /// Update with new curve styles.
   void updateSettingsCurveStyles(const CurveStyles &modelCurveStyles);
@@ -206,6 +284,9 @@ public:
   /// Update with new grid removal properties.
   void updateSettingsGridRemoval(const DocumentModelGridRemoval &modelGridRemoval);
 
+  /// Update with new guideline properties.
+  void updateSettingsGuideline (const DocumentModelGuideline &modelGuideline);
+
   /// Update with new main window properties.
   void updateSettingsMainWindow(const MainWindowModel &modelMainWindow);
 
@@ -240,7 +321,9 @@ private slots:
   void slotDigitizeAxis ();
   void slotDigitizeColorPicker ();
   void slotDigitizeCurve ();
+  void slotDigitizeGuidelines ();
   void slotDigitizePointMatch ();
+  void slotDigitizeScale ();
   void slotDigitizeSegment ();
   void slotDigitizeSelect ();
   void slotEditCopy ();
@@ -263,7 +346,10 @@ private slots:
   void slotFilePrint();
   bool slotFileSave(); /// Slot method that is sometimes called directly with return value expected
   bool slotFileSaveAs(); /// Slot method that is sometimes called directly with return value expected
+  void slotFittingWindowClosed();
+  void slotFittingWindowCurveFit(FittingCurveCoefficients, double, double, bool, bool);
   void slotGeometryWindowClosed();
+  void slotGuidelineDragged(QString, double, bool, GuidelineState);
   void slotHelpAbout();
   void slotHelpTutorial();
   void slotKeyPress (Qt::Key, bool);
@@ -277,16 +363,19 @@ private slots:
   void slotSettingsAxesChecker ();
   void slotSettingsColorFilter ();
   void slotSettingsCoords ();
-  void slotSettingsCurveAddRemove ();
+  void slotSettingsCurveList ();
   void slotSettingsCurveProperties ();
   void slotSettingsDigitizeCurve ();
   void slotSettingsExportFormat ();
   void slotSettingsGeneral ();
   void slotSettingsGridDisplay ();
   void slotSettingsGridRemoval ();
+  void slotSettingsGuideline ();  
   void slotSettingsMainWindow ();
   void slotSettingsPointMatch ();
   void slotSettingsSegments ();
+  void slotTableStatusChange ();
+  void slotTimeoutChecklistGuideWizard ();
   void slotTimeoutRegressionErrorReport ();
   void slotTimeoutRegressionFileCmdScript ();
   void slotUndoTextChanged (const QString &);
@@ -294,30 +383,31 @@ private slots:
   void slotViewGroupBackground(QAction*);
   void slotViewGroupCurves(QAction*);
   void slotViewGroupStatus(QAction*);
+  void slotViewGuidelines ();    
   void slotViewToolBarBackground ();
-  void slotViewToolBarChecklistGuide ();
   void slotViewToolBarCoordSystem ();
   void slotViewToolBarDigitize ();
-  void slotViewToolBarGeometryWindow ();
+  void slotViewWindowChecklistGuide ();
+  void slotViewWindowFitting ();
+  void slotViewWindowGeometry ();
   void slotViewToolBarSettingsViews ();
   void slotViewToolTips ();
-  void slotViewZoom16To1 ();
-  void slotViewZoom8To1 ();
-  void slotViewZoom4To1 ();
-  void slotViewZoom2To1 ();
-  void slotViewZoom1To1 ();
-  void slotViewZoom1To2 ();
-  void slotViewZoom1To4 ();
-  void slotViewZoom1To8 ();
-  void slotViewZoom1To16 ();
+  void slotViewZoomFactor (ZoomFactor);
+  void slotViewZoomFactorInt (int);
   void slotViewZoom (int);
-  void slotViewZoomFill ();
   void slotViewZoomIn ();
   void slotViewZoomInFromWheelEvent ();
   void slotViewZoomOut ();
   void slotViewZoomOutFromWheelEvent ();
 
 signals:
+
+  /// Send drag and drop regression test url
+  void signalDropRegression(QString);
+
+  /// Send wakeup signal to unit test framework when all other commands have finished executing
+  void signalGong ();
+  
   /// Send zoom selection, picked from menu or keystroke, to StatusBar.
   void signalZoom(int);
 
@@ -337,47 +427,38 @@ private:
                       Qt::DockWidgetArea dockWidgetArea);
   void applyZoomFactorAfterLoad();
   virtual void closeEvent(QCloseEvent *event);
-  void createActions();
-  void createActionsDigitize ();
-  void createActionsEdit ();
-  void createActionsFile ();
-  void createActionsHelp ();
-  void createActionsSettings ();
-  void createActionsView ();
-  void createCentralWidget ();
-  void createCommandStackShadow ();
-  void createDockableWidgets ();
-  void createHelpWindow ();
-  void createIcons();
-  void createLoadImageFromUrl ();
-  void createMenus();
-  void createNetwork();
-  void createScene ();
-  void createSettingsDialogs ();
-  void createStateContextBackground();
-  void createStateContextDigitize();
-  void createStateContextTransformation();
-  void createStatusBar();
-  void createToolBars();
-  void createTutorial();
-  void createZoomMap ();
   ZoomFactor currentZoomFactor () const;
-#ifndef OSX_RELEASE
+#if !defined(OSX_DEBUG) && !defined(OSX_RELEASE)
   void exportAllCoordinateSystemsAfterRegressionTests();
 #endif
-  QString exportFilenameFromInputFilename (const QString &fileName) const;
+  QString exportRegressionFilenameFromInputFilename (const QString &fileName) const;
   void fileExport(const QString &fileName,
                   ExportToFile exportStrategy);
+  void fileExtractImage (const QString &fileName);
   void fileImport (const QString &fileName,
                    ImportType ImportType); /// Same steps as filePaste but with import from file
   void fileImportWithPrompts (ImportType ImportType); /// Wrapper around fileImport that adds user prompt(s)
+  QString fileNameForExportOnly () const; /// File name for export-only batch mode
+  QString fileNameForExtractImageOnly () const; /// File name for extract-image-only batch mode
   void filePaste (ImportType importType); /// Same steps as fileImport but with import from clipboard
   void ghostsCreate (); /// Create the ghosts for seeing all coordinate systems at once
   void ghostsDestroy (); /// Destroy the ghosts for seeing all coordinate systems at once
+  void guidelineAddXTEnqueue (double value);
+  void guidelineAddYREnqueue (double value);
+  void guidelineMoveXTEnqueue (double valueBefore,
+                               double valueAfter);
+  void guidelineMoveYREnqueue (double valueBefore,
+                               double valueAfter);
+  void guidelineRemoveXTEnqueue (double value);
+  void guidelineRemoveYREnqueue (double value);    
+  Guidelines &guidelines (); /// Return guidelines for unit testing
+  void handleGuidelineMode();  
+  void handlerFileExtractImage (); /// Analog to slotFileExport but for image extract. Maybe converted to slot in future
   void loadCoordSystemListFromCmdMediator(); /// Update the combobox that has the CoordSystem list
   void loadCurveListFromCmdMediator(); /// Update the combobox that has the curve names.
   void loadDocumentFile (const QString &fileName);
   void loadErrorReportFile(const QString &errorReportFile);
+  void loadGuidelinesFromCmdMediator();  
   bool loadImage (const QString &fileName,
                   const QImage &image,
                   ImportType ImportType);
@@ -389,10 +470,11 @@ private:
                                 ImportType ImportType);
   void loadInputFileForErrorReport(QDomDocument &domInputFile) const;
   void loadToolTips ();
+  void loadViewsLoad (); // Apply the views settings as appropriate just after loading
+  void loadViewsSave (); // Save the current views settings just before saving
   bool maybeSave();
-  DocumentModelExportFormat modelExportOverride (const DocumentModelExportFormat &modelExportFormatBefore,
-                                                 const ExportToFile &exportStrategy,
-                                                 const QString &selectedNameFilter) const;
+  QString messageCannotReadFile (const QString &fileName) const;
+  bool modeGraph () const; // True if document is loaded and it has all graphs
   void rebuildRecentFileListForCurrentFile(const QString &filePath);
   bool saveDocumentFile(const QString &fileName);
   QString saveErrorReportFileAndExitXml (const char *comment,
@@ -402,6 +484,7 @@ private:
   void saveStartingDocumentSnapshot();
   void setCurrentFile(const QString &fileName);
   void setCurrentPathFromFile (const QString &fileName);
+  void setNonFillZoomFactor (ZoomFactor newZoomFactor);
   void setPixmap (const QString &curveSelected,
                   const QPixmap &pixmap);
   void settingsRead (bool isReset);
@@ -415,15 +498,19 @@ private:
                                      const QString &temporaryMessage,
                                      ImportType ImportType);
   void startRegressionTestErrorReport (const QString &regressionInputFile);
+  void startRegressionDropTest (const QStringList &loadStartupFiles);
   void startRegressionTestFileCmdScript ();
   void updateAfterCommandStatusBarCoords ();
   void updateChecklistGuide ();
   void updateControls (); // Update the widgets (typically in terms of show/hide state) depending on the application state.
+  void updateFittingWindow ();
   void updateGeometryWindow ();
   void updateGridLines();
+  void updateGuidelines();
   void updateHighlightOpacity();
   void updateRecentFileList();
   void updateSettingsMainWindow();
+  void updateSmallDialogs();
   void updateTransformationAndItsDependencies();
   void updateViewedCurves ();
   void updateViewsOfSettings (); // Private version gets active curve name from DigitizeContext
@@ -466,20 +553,24 @@ private:
   QActionGroup *m_groupDigitize;
   QAction *m_actionDigitizeSelect;
   QAction *m_actionDigitizeAxis;
+  QAction *m_actionDigitizeScale;
   QAction *m_actionDigitizeCurve;
+  QAction *m_actionDigitizeGuideline;  
   QAction *m_actionDigitizePointMatch;
   QAction *m_actionDigitizeColorPicker;
   QAction *m_actionDigitizeSegment;
 
   QMenu *m_menuView;
-  QAction *m_actionViewBackground;
-  QAction *m_actionViewChecklistGuide;
-  QAction *m_actionViewCoordSystem;
-  QAction *m_actionViewDigitize;
+  QAction *m_actionViewBackgroundToolBar;
+  QAction *m_actionViewChecklistGuideWindow;
+  QAction *m_actionViewCoordSystemToolBar;
+  QAction *m_actionViewDigitizeToolBar;
+  QAction *m_actionViewFittingWindow;
   QAction *m_actionViewGeometryWindow;
-  QAction *m_actionViewSettingsViews;
+  QAction *m_actionViewSettingsViewsToolBar;
   QAction *m_actionViewToolTips;
   QAction *m_actionViewGridLines;
+  QAction *m_actionViewGuidelines;
   QMenu *m_menuViewBackground;
   QActionGroup *m_groupBackground;
   QAction *m_actionViewBackgroundNone;
@@ -490,6 +581,7 @@ private:
   QAction *m_actionViewCurvesNone;
   QAction *m_actionViewCurvesSelected;
   QAction *m_actionViewCurvesAll;
+  QMenu *m_menuViewGuidelines;
   QMenu *m_menuViewStatus;
   QActionGroup *m_groupStatus;
   QAction *m_actionStatusNever;
@@ -499,28 +591,46 @@ private:
   QAction *m_actionZoomOut;
   QAction *m_actionZoomIn;
   QActionGroup *m_groupZoom;
+  QSignalMapper *m_mapperZoomFactor;
   QAction *m_actionZoomFill;
   QAction *m_actionZoom16To1;
+  QAction *m_actionZoom16To1Farther;
+  QAction *m_actionZoom8To1Closer;
   QAction *m_actionZoom8To1;
+  QAction *m_actionZoom8To1Farther;
+  QAction *m_actionZoom4To1Closer;
   QAction *m_actionZoom4To1;
+  QAction *m_actionZoom4To1Farther;
+  QAction *m_actionZoom2To1Closer;
   QAction *m_actionZoom2To1;
+  QAction *m_actionZoom2To1Farther;
+  QAction *m_actionZoom1To1Closer;
   QAction *m_actionZoom1To1;
+  QAction *m_actionZoom1To1Farther;
+  QAction *m_actionZoom1To2Closer;
   QAction *m_actionZoom1To2;
+  QAction *m_actionZoom1To2Farther;
+  QAction *m_actionZoom1To4Closer;
   QAction *m_actionZoom1To4;
+  QAction *m_actionZoom1To4Farther;
+  QAction *m_actionZoom1To8Closer;
   QAction *m_actionZoom1To8;
+  QAction *m_actionZoom1To8Farther;
+  QAction *m_actionZoom1To16Closer;
   QAction *m_actionZoom1To16;
 
   QMenu *m_menuSettings;
   QAction *m_actionSettingsAxesChecker;
   QAction *m_actionSettingsColorFilter;
   QAction *m_actionSettingsCoords;
-  QAction *m_actionSettingsCurveAddRemove;
+  QAction *m_actionSettingsCurveList;
   QAction *m_actionSettingsCurveProperties;
   QAction *m_actionSettingsDigitizeCurve;
   QAction *m_actionSettingsExport;
   QAction *m_actionSettingsGeneral;
   QAction *m_actionSettingsGridDisplay;
   QAction *m_actionSettingsGridRemoval;
+  QAction *m_actionSettingsGuideline;
   QAction *m_actionSettingsMainWindow;
   QAction *m_actionSettingsPointMatch;
   QAction *m_actionSettingsSegments;
@@ -532,7 +642,7 @@ private:
   QAction *m_actionHelpTutorial;
   QAction *m_actionHelpWhatsThis;
 
-  QVBoxLayout *m_layout;
+  QGridLayout *m_layout;
   GraphicsScene *m_scene;
   GraphicsView *m_view;
 
@@ -550,6 +660,7 @@ private:
   ViewSegmentFilter *m_viewSegmentFilter;
   QToolBar *m_toolSettingsViews;
   ChecklistGuide *m_dockChecklistGuide;
+  FittingWindow *m_dockFittingWindow;
   GeometryWindow *m_dockGeometryWindow;
 
   QComboBox *m_cmbCoordSystem;
@@ -557,7 +668,9 @@ private:
   QPushButton *m_btnShowAll;
   QToolBar *m_toolCoordSystem;
 
+#if !defined(OSX_DEBUG) && !defined(OSX_RELEASE)  
   HelpWindow *m_helpWindow;
+#endif
   TutorialDlg *m_tutorialDlg;
 
   CmdMediator *m_cmdMediator; /// Contains the Document as a private member
@@ -575,13 +688,14 @@ private:
   DlgSettingsAxesChecker *m_dlgSettingsAxesChecker;
   DlgSettingsColorFilter *m_dlgSettingsColorFilter;
   DlgSettingsCoords *m_dlgSettingsCoords;
-  DlgSettingsCurveAddRemove *m_dlgSettingsCurveAddRemove;
+  DlgSettingsCurveList *m_dlgSettingsCurveList;
   DlgSettingsCurveProperties *m_dlgSettingsCurveProperties;
-  DlgSettingsDigitizeCurve * m_dlgSettingsDigitizeCurve;
+  DlgSettingsDigitizeCurve *m_dlgSettingsDigitizeCurve;
   DlgSettingsExportFormat *m_dlgSettingsExportFormat;
   DlgSettingsGeneral *m_dlgSettingsGeneral;
   DlgSettingsGridDisplay *m_dlgSettingsGridDisplay;
   DlgSettingsGridRemoval *m_dlgSettingsGridRemoval;
+  DlgSettingsGuideline *m_dlgSettingsGuideline;  
   DlgSettingsMainWindow *m_dlgSettingsMainWindow;
   DlgSettingsPointMatch *m_dlgSettingsPointMatch;
   DlgSettingsSegments *m_dlgSettingsSegments;
@@ -598,6 +712,9 @@ private:
   QTimer *m_timerLoadStartupFiles;
   QStringList m_loadStartupFiles;
 
+  // Command line arguments with load startup files omitted
+  QStringList m_commandLineWithoutLoadStartupFiles;
+
   // Ghosts that are created for seeing all coordinate systems at once, when there are multiple coordinate systems
   Ghosts *m_ghosts;
 
@@ -616,8 +733,26 @@ private:
   // Grid lines
   GridLines m_gridLines;
 
-  // Map between zoom enumerations. This eliminates the need for a switch statement
-  QMap<ZoomFactorInitial, ZoomFactor> m_zoomMap;
+  // Guidelines that are effectively invisible until cursor gets to outermost pixels in scene (and view if zoomed out
+  // enough that there is no scroll bar)
+  Guidelines m_guidelines;
+  
+  // Map to/from/between zoom enumerations. These eliminate the need for switch statements
+  QMap<ZoomFactorInitial, ZoomFactor> m_zoomMapFromInitial;
+  QMap<ZoomFactor, QAction*> m_zoomMapToAction;
+
+  // Fitted curve. Null if not currently applicable/defined
+  FittingCurve *m_fittingCurve;
+
+  // Export the single dig file that was loaded in the command line, as enforced by parseCmdLine
+  bool m_isExportOnly;
+
+  // Extract the image from the single dig file that was loaded in the command line, as enforced by parseCmdLine
+  bool m_isExtractImageOnly;
+  QString m_extractImageOnlyExtension;
+
+  // Timer for delayed execution of ChecklistGuideWizard after call to import handler
+  QTimer *m_timerChecklistGuideWizard;
 };
 
 #endif // MAIN_WINDOW_H
